package com.sunwin.swismp.controller.sysfunction;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.swismp.bean.sysfunction.SysOperationLog;
import com.sunwin.swismp.bean.sysfunction.SysUser;
import com.sunwin.swismp.config.threadlocal.UserOrgContextHolder;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.HttpResultParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.sysfunction.LoginResult;
import com.sunwin.swismp.dto.sysfunction.SysUserOrg;
import com.sunwin.swismp.service.sysfunction.SysOperationLogService;
import com.sunwin.swismp.service.sysfunction.SysUserService;
import com.sunwin.swismp.util.HttpUtil;
import com.sunwin.swismp.util.LoggerMessageUtil;
import com.sunwin.swismp.util.mobileMessage.NeteaseImResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Map;

@RestController
@RequestMapping(value = "/login")
public class LoginController {

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private SysOperationLogService sysOperationLogService;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    private Logger logger = LoggerFactory.getLogger(LoginController.class);

    /**
     * @Description : 登录接口
     * @param params 封装的用户信息  包括 动态密码  以及 用户名
     * @param request 请求体
     * @param response 回应体
     * @Return : 会话id 以及 封装在cookie里的 会话是否过期标识 用户id 部门id
     * @Author : yangdong
     * @Date : 2020/2/28 11:47
    */
    @RequestMapping(method = RequestMethod.POST)
    @MethodDescription(description = "登录")
    public HttpResult login(@RequestBody String params, HttpServletRequest request, HttpServletResponse response) {
        HttpResult httpResult = null;
        JSONObject object = JSON.parseObject(params);
        String userName = object.get("userName").toString();
        //查询ip 是否 锁定 如果锁定 则直接不让登录
        Boolean locked = sysUserService.lock(userName);
        if(locked){
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE,HttpResponseParam.LLOGIN_LOCK);
        } else {
            String ip = HttpUtil.getIpAddr(request);
            Map map=sysUserService.login(object.get("userName").toString(),object.get("feature").toString(),object.get("vCode").toString());
            httpResult = (HttpResult) map.get("result");
            if (httpResult.getHttpCode() == HttpResponseParam.SUCCESS_CODE) { //登录成功，插入操作日志
                /*
                如果登录成功  则非法登录计数清零
                 */
                sysUserService.unlock(userName);
                SysOperationLog log = new SysOperationLog("登录", null, object.get("userName").toString(),ip);
                sysOperationLogService.insertOperationLog(log);
                HttpSession session = request.getSession();
                session.setAttribute("userId", ((LoginResult)httpResult.getData()).getUserId());
                session.setAttribute(session.getId(),object.get("userName").toString());
                session.setAttribute("userName",object.get("userName").toString());
                session.setAttribute("roleCode", String.valueOf(map.get("role_feature")));
                //session.setMaxInactiveInterval(1800);
                session.setMaxInactiveInterval(18000);//5小时
                //session.setMaxInactiveInterval(60*60*24);//一天
                Cookie cookiePresent=new Cookie("sessionIsPresent","true");
                cookiePresent.setPath("/");//cookie设置会话是否过期标识
                cookiePresent.setHttpOnly(false);
                Cookie cookieUserId=new Cookie("u_feature",String.valueOf(map.get("u_feature")));
                cookieUserId.setPath("/");//cookie设置userid
                Cookie cookieOrgId=new Cookie("org_feature",String.valueOf(map.get("org_feature")));
                Cookie cookieRoleCode = new Cookie("role_feature",String.valueOf(map.get("role_feature")));
                Cookie cookieRoleName = new Cookie("role_name",String.valueOf(map.get("role_name")));
                cookieRoleCode.setPath("/");//cookie设置role_feature
                cookieRoleName.setPath("/");//cookie设置role_name
                cookieOrgId.setPath("/");//cookie设置orgid
                response.addCookie(cookiePresent);
                response.addCookie(cookieUserId);
                response.addCookie(cookieOrgId);
                response.addCookie(cookieRoleCode);
                response.addCookie(cookieRoleName);
                SysUserOrg userOrg=new SysUserOrg(Integer.parseInt(String.valueOf(map.get("u_feature"))),Integer.parseInt(String.valueOf(map.get("org_feature"))),String.valueOf(map.get("role_feature")));
                UserOrgContextHolder.setUserOrg(userOrg);//将用户部门信息放入线程全局变量中
            } else {
                /*
                记录该ip的登录次数  如果达到5次 则直接提示  并更改redis状态 否则 只是记录
                 */
                Map<String,String> lockedStart = sysUserService.updateFailCount(userName);
                if(lockedStart.get("locakedStart").equals("true")){
                    httpResult = new HttpResult(HttpResponseParam.FAIL_CODE,HttpResponseParam.LLOGIN_LOCK);
                } else {
                    String countRemain = "当前您还剩余"+lockedStart.get("countRemain")+"次连续的登录机会";
                    String des = httpResult.getMsg();
                    if(des == null || des.trim().equals("")){
                        httpResult.setMsg(countRemain);
                    } else {
                        httpResult.setMsg(httpResult.getMsg()+","+countRemain);
                    }
                }
            }
        }
        LoggerMessageUtil.successfulMsg(logger,"登录");
        return httpResult;
    }


    /**
     * 给手机号码发送验证码
     * @param userName
     * @return
     * @throws Exception
     */
    @RequestMapping(method = RequestMethod.GET, value = "/captcha")
    @MethodDescription(description = "发送登录验证码")
    public HttpResult getSmsCaptcha(@RequestParam String userName) throws Exception{
        HttpResult httpResult = null;
        NeteaseImResponse response = null;
        SysUser sysUser = sysUserService.queryUserByName(userName);
        //TODO 验证码暂时不处理， 默认 111111 ，启用替换下段注释的代码
        if (sysUser != null && sysUser.getMobilePhone() !=null && sysUser.getMobilePhone() != "" ){
            sysUserService.setCodeRedis("swismp_login_telephone_"+sysUser.getMobilePhone(), String.valueOf(111111), HttpResultParam.VERIFICATION_CODE_TIMEOUT);
            //redisTemplate.opsForValue().set( "swurt_login_telephone_:"+telephone, String.valueOf(response.getObj()), HttpResultParam.VERIFICATION_CODE_TIMEOUT,TimeUnit.MINUTES);
            httpResult = new HttpResult(HttpResultParam.SUCCESS_CODE, "发送成功,测试阶段，默认为 111111");
        }else {
            httpResult = new HttpResult(HttpResultParam.FAIL_CODE, "账号或账号手机号不存在");
        }
        /*
        if (sysUser != null && sysUser.getMobilePhone() !=null && sysUser.getMobilePhone() != "" ){
            response = MobileMessageSendUtil.sendMsg(sysUser.getMobilePhone(), "swbd_login_templateid");
        }else {
            httpResult = new HttpResult(HttpResultParam.FAIL_CODE, "账号或账号手机号不存在");
        }

        if (response != null && response.getCode() == 200) {
            sysUserService.setCodeRedis("login_telephone_"+sysUser.getMobilePhone(), String.valueOf(response.getObj()), HttpResultParam.VERIFICATION_CODE_TIMEOUT);
            //redisTemplate.opsForValue().set( "login_telephone:"+telephone, String.valueOf(response.getObj()), HttpResultParam.VERIFICATION_CODE_TIMEOUT,TimeUnit.MINUTES);
            httpResult = new HttpResult(HttpResultParam.SUCCESS_CODE, "发送成功");
        } else if (response != null && response.getCode() == 416){
            httpResult = new HttpResult(HttpResultParam.FAIL_CODE, "今日该账号手机验证码获取次数已达上限");
        }else {
            if (httpResult == null) {
                httpResult = new HttpResult(HttpResultParam.FAIL_CODE, "发送失败");
            }
        }
        */
        LoggerMessageUtil.successfulMsg(logger,"发送登录验证码");
        return httpResult;
    }
}
