package com.sunwin.swismp.controller.sysfunction;

import com.sunwin.swismp.bean.TreeNode;
import com.sunwin.swismp.bean.sysfunction.SysMenu;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysMenuDTO;
import com.sunwin.swismp.service.sysfunction.SysMenuService;
import com.sunwin.swismp.util.LoggerMessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @Description: 系统菜单管理接口
 * @Author: fengyong
 * @CreateDate: 2022/2/15 09:16
 * @Remark:
 * @Version: 1.0
 */
@RestController
@RequestMapping(value = "/menu")
public class SysMenuController {
    private Logger logger = LoggerFactory.getLogger(SysMenuController.class);
    @Autowired
    private SysMenuService sysMenuService;

    /**
     * 查询指定菜单明细
     * @param menuId
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/queryMenu")
    public HttpResult queryMenuDetail(@RequestParam("menuId") Integer menuId) {
        HttpResult httpResult = null;
        try {
            SysMenuDTO sysMenu = sysMenuService.selectMenuById(menuId);
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE,HttpResponseParam.SUCCESS_MSG,sysMenu);
            LoggerMessageUtil.successfulMsg(logger,"查询指定菜单明细");
        }catch (Exception e) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE,HttpResponseParam.FAIL_MSG);
            LoggerMessageUtil.errorMsg(logger,"查询指定菜单明细",e);
        }
        return httpResult;
    }
    /**
     * 查询次级菜单列表
     * @param menuId
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/queryMenu/secondary")
    public HttpResult queryMenuSecondary(@RequestParam("menuId") Integer menuId,
                                         @RequestParam(value = "resourceType",required = false) Integer resourceType,
                                         @RequestParam(name = "isEnable", required = false) Integer isEnable,
                                         @RequestParam(value = "keywords",required = false) String keywords,
                                         @RequestParam(name = "pageNum") Integer pageNum,
                                         @RequestParam(name = "pageSize") Integer pageSize) {
        HttpResult httpResult = null;
        try {
            PageResult pageResult = sysMenuService.queryMenuSecondary(menuId, resourceType, isEnable, keywords, pageNum, pageSize);
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE,HttpResponseParam.SUCCESS_MSG,pageResult);
            LoggerMessageUtil.successfulMsg(logger,"查询次级菜单列表");
        }catch (Exception e) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE,HttpResponseParam.FAIL_MSG);
            LoggerMessageUtil.errorMsg(logger,"查询次级菜单列表",e);
        }
        return httpResult;
    }

    /**
     * @Description : 添加、修改资源（菜单）
     * @param sysMenu 封装的菜单信息
     * @Return : 提交是否成功
     * @Author : fengyong
     * @Date : 2022/2/15 13:37
     */
    @RequestMapping(method = RequestMethod.POST, value = "/save")
    @MethodDescription(description = "保存系统资源信息")
    public HttpResult menuSave(@RequestBody SysMenu sysMenu, HttpServletResponse response) {
        HttpResult httpResult = null;
        httpResult = sysMenuService.menuSave(sysMenu);
        LoggerMessageUtil.successfulMsg(logger,"保存系统资源信息");
        return httpResult;
    }

    /**
     * @Description : 修改启用状态
     * @Return :
     */
    @RequestMapping(method = RequestMethod.POST, value = "/update/enabled")
    @MethodDescription(description = "修改启用状态")
    public HttpResult updateEnabled(@RequestBody SysMenu sysMenu) {
        HttpResult httpResult = null;
        Integer index = sysMenuService.updateEnabled(sysMenu);
        if (index > 0) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "修改启用状态");
        return httpResult;
    }

    /**
     * @Description : 删除资源
     * @Return :
     */
    @RequestMapping(method = RequestMethod.GET, value = "/delete")
    @MethodDescription(description = "删除资源")
    public HttpResult deleteMenu(@RequestParam(value = "menuId") Integer menuId) {
        HttpResult httpResult = null;
        Integer index = sysMenuService.deleteMenu(menuId);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_DELETE_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_EXIST_CHILDREN_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "删除资源");
        return httpResult;
    }

    /**
     * @Description : 系统资源菜单树
     * @Return :
     */
    @GetMapping(value = "/sysMenuTree")
    @MethodDescription(description = "系统资源菜单树")
    public HttpResult sysMenuTree() {
        HttpResult httpResult = null;
        List<TreeNode> root = sysMenuService.sysMenuTree();
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, root);
        LoggerMessageUtil.successfulMsg(logger, "系统资源菜单树");
        return httpResult;
    }
}
