package com.sunwin.swismp.controller.sysfunction;

import com.sunwin.swismp.bean.TreeNode;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysOrgDTO;
import com.sunwin.swismp.service.sysfunction.SysOrgService;
import com.sunwin.swismp.util.LoggerMessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Copyright (c) 12/20/2021 All Rights Reserved By SUNWIN
 * <p>
 * Package: com.sunwin.swismp.controller.sysfunction
 * FileName: SysOrgController
 *
 * @author duanmin
 * @description
 * @created 2/11/2022 - 5:29 PM
 * @last Modified
 * @history
 */
@RestController
@RequestMapping(value = "/org")
public class SysOrgController {
    private Logger logger = LoggerFactory.getLogger(SysOrgController.class);

    @Autowired
    private SysOrgService sysOrgService;

    /**
     * @Description : 新增组织
     * @Return :
     */
    @PostMapping(value = "/addSysOrg", produces = {"application/json;charset=UTF-8"})
    @MethodDescription(description = "新增组织")
    public HttpResult addSysOrg(@RequestBody SysOrgDTO sysOrgDTO) {
        HttpResult httpResult = null;
        Integer index = sysOrgService.addSysOrg(sysOrgDTO);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_INSERT_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_ADD_DUPLICATTED_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_INSERT_MSG);
        }

        LoggerMessageUtil.successfulMsg(logger, "新增组织");
        return httpResult;
    }

    /**
     * @Description : 删除组织
     * @Return :
     */
    @GetMapping(value = "/delSysOrg")
    @MethodDescription(description = "删除组织")
    public HttpResult delSysOrg(@RequestParam(value = "sysOrgId") Integer sysOrgId) {
        HttpResult httpResult = null;
        Integer index = sysOrgService.delSysOrg(sysOrgId);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_DELETE_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_EXIST_CHILDREN_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "删除组织");
        return httpResult;
    }

    /**
     * @Description : 修改组织
     * @Return :
     */
    @PostMapping(value = "/editSysOrg", produces = {"application/json;charset=UTF-8"})
    @MethodDescription(description = "修改组织")
    public HttpResult editSysOrg(@RequestBody SysOrgDTO SysOrgDTO) {
        HttpResult httpResult = null;
        Integer index = sysOrgService.editSysOrg(SysOrgDTO);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_DUPLICATTED_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }

        LoggerMessageUtil.successfulMsg(logger, "修改组织");
        return httpResult;
    }

    /**
     * @Description : 查看组织
     * @Return :
     */
    @GetMapping(value = "/viewSysOrg")
    @MethodDescription(description = "查看组织")
    public HttpResult viewSysOrg(@RequestParam(value = "sysOrgId") String sysOrgId) {
        HttpResult httpResult = null;
        SysOrgDTO sysOrgDTO = sysOrgService.viewSysOrg(sysOrgId);
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, sysOrgDTO);
        LoggerMessageUtil.successfulMsg(logger, "查看组织");
        return httpResult;
    }

    /**
     * @Description : 修改启用状态
     * @Return :
     */
    @GetMapping(value = "/enableSysOrg")
    @MethodDescription(description = "修改启用状态")
    public HttpResult enableSysOrg(@RequestParam(value = "sysOrgId") Integer sysOrgId,
                                   @RequestParam(value = "enabled") Integer enabled) {
        HttpResult httpResult = null;
        Integer index = sysOrgService.enableSysOrg(sysOrgId, enabled);
        if (index > 0) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "修改启用状态");
        return httpResult;
    }

    /**
     * @Description : 查询组织-分页
     * @Return :
     */
    @GetMapping(value = "/querySysOrgPage")
    @MethodDescription(description = "查询组织-分页")
    public HttpResult querySysOrgPage(@RequestParam(value = "parentOrgId", required = false) String parentOrgId,
                                      @RequestParam(value = "orgUserNum", required = false) String orgUserNum,
                                      @RequestParam(value = "isEnabled", required = false) String isEnabled,
                                      @RequestParam(value = "keywords") String keywords,
                                      @RequestParam(value = "beginUpdateTime") String beginUpdateTime,
                                      @RequestParam(value = "endUpdateTime") String endUpdateTime,
                                      @RequestParam(value = "pageNum") Integer pageNum,
                                      @RequestParam(value = "pageSize") Integer pageSize) {
        HttpResult httpResult = null;
        PageResult pageResult = sysOrgService.querySysOrgPage(parentOrgId, orgUserNum, isEnabled, keywords,
                beginUpdateTime, endUpdateTime, pageNum, pageSize);
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, pageResult);
        LoggerMessageUtil.successfulMsg(logger, "查询组织-分页");
        return httpResult;
    }

    /**
     * @Description : 组织树
     * @Return :
     */
    @GetMapping(value = "/sysOrgTree")
    @MethodDescription(description = "组织树")
    public HttpResult sysOrgTree() {
        HttpResult httpResult = null;
        List<TreeNode> root = sysOrgService.sysOrgTree();
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, root);
        LoggerMessageUtil.successfulMsg(logger, "组织树");
        return httpResult;
    }
}
