package com.sunwin.swismp.controller.sysfunction;


import com.alibaba.fastjson.JSONObject;
import com.sunwin.swismp.bean.sysfunction.SysRole;
import com.sunwin.swismp.config.threadlocal.UserOrgContextHolder;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysRoleDTO;
import com.sunwin.swismp.dto.sysfunction.SysUserOrg;
import com.sunwin.swismp.service.sysfunction.SysRoleMenuPermissionsService;
import com.sunwin.swismp.service.sysfunction.SysRoleService;
import com.sunwin.swismp.util.LoggerMessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
* @Description: 系统角色接口
* @Author: zc
* @CreateDate: 2018/11/16 17:16
* @Remark:
* @Version: 1.0
*/
@RestController
@RequestMapping(value = "/role")
public class SysRoleController {

    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysRoleMenuPermissionsService sysRoleMenuPermissionsService;

    private Logger logger = LoggerFactory.getLogger(SysRoleController.class);


    /**
     * @Description : 角色下拉框参数
     * @Return : 角色信息列表
     * @Author : yangdong
     * @Date : 2020/2/28 13:36
    */
    @RequestMapping(method = RequestMethod.GET,value = "/box")
    @MethodDescription(description = "角色下拉框参数")
    public HttpResult queryRoleBox() {
        HttpResult httpResult = null;
        //获取当前用户的roleid
        SysUserOrg userOrg = UserOrgContextHolder.getUserOrg();
        String roleCode = null;
        if(userOrg != null){
            roleCode = userOrg.getRoleCode();
        }
        /*
        判断是否是管理员  或是 普通管理员
         */
        //TODO:没提需求，暂不加权限，若加权限，注释此行
        roleCode = "10001";
        if(roleCode == null){
            httpResult = new HttpResult(HttpResponseParam.NO_DATA_CODE, HttpResponseParam.NO_DATA_MSG);
        } else {
            List<SysRole> roles = sysRoleService.selectAllByRoleCode(roleCode);
            List<Map> list = new ArrayList<>();
            for (SysRole sysRole:roles){
                    Map map = new HashMap<>();
                    map.put("value",sysRole.getRoleId());
                    map.put("label",sysRole.getRoleName());
                    list.add(map);
            }
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG,list);
        }
        return httpResult;
    }

    /**
     * @Description : 删除角色
     * @param SysRole 角色id
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 13:37
    */
    @RequestMapping(method = RequestMethod.DELETE,value = "/delete")
    @MethodDescription(description = "删除角色")
    public HttpResult deleteRole(@RequestBody SysRole sysRole) {
        HttpResult httpResult = null;
        int index = sysRoleService.deleteRoleByRoleId(sysRole.getRoleId());
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_DELETE_MSG);
        } else if(index == 2){
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_EXIST_USER_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_DELETE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger,"删除角色");
        return httpResult;
    }


    /**
     *  角色-分页条件查询
     * @param roleName
     * @param roleCode
     * @param isEnable
     * @param page
     * @param size
     * @param response
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/list")
    @MethodDescription(description = "查询角色信息列表")
    public HttpResult rolesInfo(@RequestParam(name = "roleName", required = false) String roleName,
                                @RequestParam(name = "roleCode", required = false) String roleCode,
                                @RequestParam(name = "isEnable", required = false) Integer isEnable,
                                @RequestParam(value = "keywords",required = false) String keywords,
                                @RequestParam(name = "pageNum") Integer page,
                                @RequestParam(name = "pageSize") Integer size,
                                HttpServletResponse response) {
        HttpResult httpResult = null;
        PageResult pageResult = new PageResult();
        List<SysRoleDTO> list = sysRoleService.selectPage(roleName,roleCode,isEnable, keywords,page,size);
        pageResult.setData(list);
        Integer count = sysRoleService.selectPageCount(roleName, roleCode, isEnable, keywords);
        pageResult.setCount(Long.valueOf(count));
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG,pageResult);
        LoggerMessageUtil.successfulMsg(logger,"查询角色信息列表");
        return httpResult;
    }

    /**
     * @Description : 添加、修改角色
     * @param sysRole 封装的角色信息
     * @Return : 提交是否成功
     * @Author : yangdong
     * @Date : 2020/2/28 13:37
     */
    @RequestMapping(method = RequestMethod.POST, value = "/save")
    @MethodDescription(description = "保存角色信息")
    public HttpResult rolesSave(@RequestBody SysRole sysRole, HttpServletResponse response) {
        HttpResult httpResult = null;
        httpResult = sysRoleService.roleSave(sysRole);
        LoggerMessageUtil.successfulMsg(logger,"保存角色信息");
        return httpResult;
    }

    /**
     * @Description : 查询角色权限
     * @param roleId
     * @param response
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 13:45
    */
    @RequestMapping(method = RequestMethod.GET, value = "/queryMenus")
    @MethodDescription(description = "查询角色权限")
    public HttpResult queryRoleMenu(@RequestParam("roleId") Integer roleId, HttpServletResponse response) {
        HttpResult httpResult = null;
        Map map = sysRoleMenuPermissionsService.selectByRoleId(roleId,"all");//type : all 查询全部 ; self 查询已有的
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG,map);
        LoggerMessageUtil.successfulMsg(logger,"查询角色权限");
        return httpResult;
    }

    /**
     * 查询角色权限(查询已分配的)
     * @param roleId
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/queryMenusHave")
    public HttpResult queryRoleMenuSelf(@RequestParam("roleId") Integer roleId, HttpServletResponse response) {
        HttpResult httpResult = null;
        try {
            Map map = sysRoleMenuPermissionsService.selectByRoleId(roleId,"self");//type : all 查询全部 ; self 查询已有的
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE,HttpResponseParam.SUCCESS_MSG,map);
            LoggerMessageUtil.successfulMsg(logger,"查询已分配角色权限");
        }catch (Exception e) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE,HttpResponseParam.FAIL_MSG);
            LoggerMessageUtil.errorMsg(logger,"查询已分配角色权限",e);
        }
        return httpResult;
    }
    /**
     * @Description : 修改角色权限
     * @param object 角色信息
     * @param response
     * @Return : 是否成功
     * @Author : yangdong
     * @Date : 2020/2/28 13:45
    */
    @RequestMapping(method = RequestMethod.PUT, value = "/menus")
    @MethodDescription(description = "修改已分配角色权限")
    public HttpResult updateRoleMenu(@RequestBody JSONObject object, HttpServletResponse response) {
        HttpResult httpResult = null;
        Integer roleId = object.getInteger("roleId");
        List<Integer> menuIds = object.getJSONArray("menuIds").toJavaList(Integer.class);
        boolean success = sysRoleService.updateRoleMenu(roleId, menuIds);
        if (success) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger,"修改已分配角色权限");
        return httpResult;
    }
}
