package com.sunwin.swismp.controller.sysfunction;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.swismp.bean.sysfunction.SysUser;
import com.sunwin.swismp.config.threadlocal.UserOrgContextHolder;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysUserDTO;
import com.sunwin.swismp.dto.sysfunction.SysUserManage;
import com.sunwin.swismp.dto.sysfunction.SysUserOrg;
import com.sunwin.swismp.service.sysfunction.SysUserService;
import com.sunwin.swismp.util.LoggerMessageUtil;
import com.sunwin.swismp.util.Md5Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/user")
public class SysUserController {

    @Autowired
    private SysUserService sysUserService;
    /*@Autowired
    private SysRoleService sysRoleService;*/

    private Logger logger = LoggerFactory.getLogger(SysUserController.class);

    /**
     * @Description : 查看用户信息
     * @Return : 封装的用户信息
     */
    @RequestMapping(method = RequestMethod.GET, value = "/info")
    @MethodDescription(description = "查询用户信息")
    public HttpResult queryUserInfo(@RequestParam(value = "userId") Integer userId) {
        HttpResult httpResult = null;
        SysUserManage sysUserManage = sysUserService.selectUserManageById(userId);
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, "成功", sysUserManage);
        LoggerMessageUtil.successfulMsg(logger, "查询用户信息");
        return httpResult;
    }

    /**
     * 用户修改密码
     *
     * @param json
     * @param session
     * @param response
     * @return
     */
    @RequestMapping(method = RequestMethod.PUT, value = "/info/pwd")
    @MethodDescription(description = "修改用户密码")
    public HttpResult updateUserPwd(@RequestBody JSONObject json, HttpSession session, HttpServletResponse response) {
        HttpResult httpResult = null;
        Integer userId = (Integer) session.getAttribute("userId");
        SysUser sysUser = sysUserService.queryUserById(userId);
        String oldPwd = json.getString("oldPassword");
        String newPwd = json.getString("newPassword");
        if (sysUser.getUserPwd().equals(oldPwd)) { //验证原密码
            if (!sysUser.getUserPwd().equals(newPwd)) {
                int flag = sysUserService.updatePwd(userId, newPwd, oldPwd);
                if (flag > 0) {
                    httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG, flag);
                } else {
                    httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
                }
            } else {
                httpResult = new HttpResult(HttpResponseParam.LOGIN_PASSWORD_CODE, "修改的新密码不能与原密码一致");
            }
        } else {
            httpResult = new HttpResult(HttpResponseParam.LOGIN_PASSWORD_CODE, "原密码错误");
        }
        LoggerMessageUtil.successfulMsg(logger, "修改用户密码");
        return httpResult;
    }

    /**
     * 用户修改手机号码
     *
     * @param json
     * @param session
     * @param response
     * @return
     */
    @RequestMapping(method = RequestMethod.PUT, value = "/info/telephone")
    @MethodDescription(description = "修改用户手机号")
    public HttpResult updateUserTelephone(@RequestBody JSONObject json, HttpSession session, HttpServletResponse response) {
        HttpResult httpResult = null;
        Integer userId = (Integer) session.getAttribute("userId");
        //比对原号码的验证码
        //TODO: 暂时不用验证码
        //boolean success = sysUserService.updateTelephone(userId, json.getString("newTelephone"), json.getString("newCode"));
        boolean success = sysUserService.updateTelephone(userId, json.getString("newTelephone"));
        if (success) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "修改用户手机号");
        return httpResult;
    }

    /**
     * 用户修改邮箱
     *
     * @param json     用户邮箱信息
     * @param session  用户信息
     * @param response 响应内容
     * @return
     */
    @RequestMapping(method = RequestMethod.PUT, value = "/info/email")
    @MethodDescription(description = "修改用户邮箱")
    public HttpResult updateUserEmail(@RequestBody JSONObject json, HttpSession session, HttpServletResponse response) {
        HttpResult httpResult = null;
        Integer userId = (Integer) session.getAttribute("userId");
        boolean success = sysUserService.updateEmail(userId, json.getString("email"));
        if (success) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "修改用户邮箱");
        return httpResult;
    }


    /**
     * 添加用户
     *
     * @param user
     * @return
     */
    @RequestMapping(method = RequestMethod.POST)
    @MethodDescription(description = "添加用户")
    public HttpResult register(@RequestBody SysUser user) {  //注册
        HttpResult httpResult = null;
        if (user.getIsEnable() == null) {
            user.setIsEnable(1);//默认可用
        }
        user.setCreateTime(new Date());//创建时间
        httpResult = sysUserService.addUser(user);
        LoggerMessageUtil.successfulMsg(logger, "添加用户邮箱");
        return httpResult;
    }

    /**
     * @Description : 新增用户
     * @Return :
     */
    @PostMapping(value = "/addSysUser", produces = {"application/json;charset=UTF-8"})
    @MethodDescription(description = "新增用户")
    public HttpResult addSysUser(@RequestBody SysUserDTO sysUserDTO) {
        HttpResult httpResult = null;
        Integer index = sysUserService.addSysUser(sysUserDTO);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_INSERT_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_ADD_DUPLICATTED_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_INSERT_MSG);
        }

        LoggerMessageUtil.successfulMsg(logger, "新增用户");
        return httpResult;
    }


    /**
     * 更新用户信息
     *
     * @param user
     * @return
     */
    @PostMapping(value = "/updateUser", produces = {"application/json;charset=UTF-8"})
    @MethodDescription(description = "更新用户信息")
    public HttpResult updateUser(@RequestBody SysUser user) { //更新
        HttpResult httpResult = null;
        Integer index = sysUserService.editUser(user);
        if (index == 1) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
        } else if (index == 2) {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_DUPLICATTED_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "更新用户信息");
        return httpResult;
    }

    /**
     * @Description : 查询用户-分页
     * @Return :
     */
    @GetMapping(value = "/querySysUserPage")
    @MethodDescription(description = "查询用户-分页")
    public HttpResult querySysOrgPage(@RequestParam(value = "orgId", required = false) Integer orgId,
                                      @RequestParam(value = "orgType", required = false) String orgType,
                                      @RequestParam(value = "roleId", required = false) String roleId,
                                      @RequestParam(value = "realName", required = false) String realName,
                                      @RequestParam(value = "isEnabled", required = false) String isEnabled,
                                      @RequestParam(value = "userName", required = false) String userName,
                                      @RequestParam(value = "beginUpdateTime", required = false) String beginUpdateTime,
                                      @RequestParam(value = "endUpdateTime", required = false) String endUpdateTime,
                                      @RequestParam(value = "pageNum") Integer pageNum,
                                      @RequestParam(value = "pageSize") Integer pageSize) {
        HttpResult httpResult = null;
        PageResult pageResult = sysUserService.querySysUserPage(orgId, orgType, roleId, realName, isEnabled, userName,
                beginUpdateTime, endUpdateTime, pageNum, pageSize);
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, pageResult);
        LoggerMessageUtil.successfulMsg(logger, "查询用户-分页");
        return httpResult;
    }

    /**
     * 用户信息详情
     *
     * @param userId
     * @return
     */
    @GetMapping(value = "/userView", produces = {"application/json;charset=UTF-8"})
    @MethodDescription(description = "用户信息详情")
    public HttpResult userView(@RequestParam(value = "userId", required = false) Integer userId) {
        HttpResult httpResult = null;
        SysUserDTO sysUserDTO = sysUserService.userView(userId);
        httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, sysUserDTO);
        LoggerMessageUtil.successfulMsg(logger, "用户信息详情");
        return httpResult;
    }

    /**
     * 查看 部门 用户 权限 桑吉图
     *
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/view/relation")
    @MethodDescription(description = "查看部门用户权限桑吉图")
    public HttpResult updateUserOrgRoleRelationView() { //更新
        HttpResult httpResult = null;
        Map result = sysUserService.queryUserOrgRoleView();
        if (result != null && result.size() > 0) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, result);
        } else {
            httpResult = new HttpResult(HttpResponseParam.NO_DATA_CODE, HttpResponseParam.NO_DATA_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "查看部门用户权限桑吉图");
        return httpResult;
    }

    /**
     * 删除，暂停，启用
     *
     * @param userId
     * @param type
     * @return
     */
    @RequestMapping(method = RequestMethod.GET, value = "/operation")
    @MethodDescription(description = "删除、暂停、启用用户")
    public HttpResult operateUser(@RequestParam("userId") Integer userId,
                                  @RequestParam(value = "type", required = false) Integer type,
                                  @RequestParam(value = "enabled", required = false) Integer enabled) {
        HttpResult httpResult = null;
        int result = 0;
        if (type == 0) { //启用
            result = sysUserService.enableUser(userId, enabled);
        } else if (type == 1) { //暂停，注销
            result = sysUserService.logoff(userId);
        } else if (type == 2) { //物理删除
            result = sysUserService.deleteUser(userId);
        }
        if (result > 0) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_CHANGE_MSG);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_CHANGE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "删除、暂停、启用用户");
        return httpResult;
    }

    /**
     * @param queryType  查找方式  1 用户名  2 电话号码  3 email
     * @param queryValue 查找值
     * @param pageNo     第几页
     * @param pageSize   显示条数
     * @Description : 查询所有用户信息
     * @Return : 用户信息列表
     * @Author : yangdong
     * @Date : 2020/2/28 13:29
     */
    @RequestMapping(method = RequestMethod.GET)
    @MethodDescription(description = "查询系统用户管理信息")
    public HttpResult querySysUserManage(@RequestParam(value = "queryType") Integer queryType, @RequestParam(value = "queryValue", required = false) String queryValue, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        HttpResult httpResult = null;
        //获取当前用户的orgId
        SysUserOrg userOrg = UserOrgContextHolder.getUserOrg();
        Integer orgId = null;
        String roleCode = null;
        if (userOrg != null) {
            orgId = userOrg.getOrgId();
            roleCode = userOrg.getRoleCode();
        }
        List<SysUserManage> list = null;
        String userName = null;
        String phone = null;
        String email = null;
        if (queryType == 1) { //通过用户名查询
            userName = queryValue;
        } else if (queryType == 2) { //通过电话号码查询
            phone = queryValue;
        } else if (queryType == 3) { //通过email查询
            email = queryValue;
        }
        pageNo = (pageNo - 1) * pageSize;
        Integer count = 0;
        Map map = new HashMap();
        if (orgId == null || roleCode == null) {
            httpResult = new HttpResult(HttpResponseParam.NO_DATA_CODE, HttpResponseParam.NO_DATA_MSG, map);
        } else {
            if (queryType == 0) { //查询全部
                if (!roleCode.equals("10001")) {
                    list = sysUserService.selectAllUserManage(queryValue, pageNo, pageSize, orgId);
                    count = sysUserService.selectAllUserManageCount(queryValue, orgId);
                } else {
                    list = sysUserService.selectAllUserManage(queryValue, pageNo, pageSize, -10001);
                    count = sysUserService.selectAllUserManageCount(queryValue, -10001);
                }

            } else {
                if (!roleCode.equals("10001")) {
                    list = sysUserService.selectUserManageByParams(userName, phone, email, pageNo, pageSize, orgId);
                    count = sysUserService.selectUserManageByParamsCount(userName, phone, email, orgId);
                } else {
                    list = sysUserService.selectUserManageByParams(userName, phone, email, pageNo, pageSize, -10001);
                    count = sysUserService.selectUserManageByParamsCount(userName, phone, email, -10001);
                }

            }
            map.put("count", count);
            map.put("list", list);
            if (list != null && !list.isEmpty()) {
                httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_MSG, JSON.toJSONStringWithDateFormat(map, "yyyy-MM-dd HH:mm:ss"));
            } else {
                httpResult = new HttpResult(HttpResponseParam.NO_DATA_CODE, HttpResponseParam.NO_DATA_MSG, map);
            }
        }
        LoggerMessageUtil.successfulMsg(logger, "查询系统用户管理信息");
        return httpResult;
    }

    /**
     * 修改密码
     *
     * @param userPwdInfo
     * @return
     */
    @RequestMapping(method = RequestMethod.PUT, value = "/reset")
    @MethodDescription(description = "修改密码")
    public HttpResult reSetUserPwd(@RequestBody String userPwdInfo) {
        HttpResult httpResult = null;
        JSONObject object = JSON.parseObject(userPwdInfo);
        Integer userId = Integer.valueOf(object.get("userId").toString());
        String oldPwd = object.get("feature").toString();
        String newPwd = object.get("newPwd").toString();
        SysUser sysUser = sysUserService.queryUserById(userId);
        if (sysUser != null && Md5Util.processUserId(oldPwd, sysUser, Md5Util.TimeMd5Type.DATE)) { //验证原密码
            if (!sysUser.getUserPwd().equals(newPwd)) {
                int flag = sysUserService.updatePwd(userId, newPwd, oldPwd);
                if (flag > 0) {
                    httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG, flag);
                } else {
                    httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
                }
            } else {
                httpResult = new HttpResult(HttpResponseParam.LOGIN_PASSWORD_CODE, "修改的新密码不能与原密码一致");
            }
        } else {
            httpResult = new HttpResult(HttpResponseParam.LOGIN_PASSWORD_CODE, "原密码错误");
        }
        LoggerMessageUtil.successfulMsg(logger, "修改密码");
        return httpResult;
    }


    /**
     * 初始化密码为123456
     *
     * @param sysUser
     * @return
     */
    @RequestMapping(method = RequestMethod.POST, value = "/intial")
    @MethodDescription(description = "重置密码")
    public HttpResult intialUserPwd(@RequestBody SysUser sysUser) {
        HttpResult httpResult = null;
        int flag = sysUserService.resetPwd(sysUser.getUserId(), "123456");
        if (flag > 0) {
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG, flag);
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_UPDATE_MSG);
        }
        LoggerMessageUtil.successfulMsg(logger, "重置密码");
        return httpResult;
    }


}
