package com.sunwin.swismp.exception;

import com.alibaba.fastjson.JSON;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.constant.MethodDescription;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.util.LoggerMessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * <br>
 * Copyright (c) 1/21/2020 All Rights Reserved By SUNWIN<br>
 * <br>
 * Package: com.sunwin.swbd.exception <br>
 * FileName: CustomExceptionResolver <br>
 * <br>
 * 自定义异常统一处理
 *
 * @author duanmin
 * @created 1/21/2020-10:29 AM
 * @last Modified
 * @history
 */
@Configuration
public class CustomExceptionResolver implements HandlerExceptionResolver {
    private static Logger logger = LoggerFactory.getLogger(CustomExceptionResolver.class);

    @Override
    public ModelAndView resolveException(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse,
                                         Object handler, Exception ex) {


        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.setStatus(HttpStatus.OK.value());
        Map<String, String> body = this.getBody(ex);
        try {
            HttpResult httpResult = new HttpResult(HttpResponseParam.FAIL_MSG);
            String handlerStr = handler.toString();
            int len = handlerStr.split(" ")[2].split("\\(")[0].split("\\.").length - 1;
            String methodName = handlerStr.split(" ")[2].split("\\(")[0].split("\\.")[len];
            String clazzName = handlerStr.split(" ")[2].split("\\(")[0].split("\\." + methodName)[0];
            Class clazz = Class.forName(clazzName);
            Method[] methods = clazz.getMethods();
            for (Method method : methods) {
                if (method.getName().equals(methodName)) {
                    MethodDescription methodDescription = method.getAnnotation(MethodDescription.class);
                    if (methodDescription != null) {
                        httpResult.setDescription(methodDescription.description());
                    } else {
                        httpResult.setDescription(body.get("message"));
                    }
                    Logger logger1 = LoggerFactory.getLogger(clazzName);
                    LoggerMessageUtil.errorMsg(logger1, methodName, httpResult.getDescription(), ex);
                    httpServletResponse.getWriter().write(JSON.toJSONString(httpResult));
                }
            }

        } catch (Exception e) {
            LoggerMessageUtil.errorMsg(logger, HttpResponseParam.FAIL_MSG, e);
        }
        return new ModelAndView();
    }

    /**
     * 获取错误编码
     *
     * @param request
     * @return
     */
    private HttpStatus getStatus(HttpServletRequest request) {
        Integer statusCode = (Integer) request.getAttribute("javax.servlet.error.status_code");
        if (statusCode == null) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
        try {
            return HttpStatus.valueOf(statusCode);
        } catch (Exception ex) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
    }

    /**
     * <p>
     * <code>getBody</code>根据exception获取返回信息
     * </p>
     *
     * @param ex
     * @return
     */
    private Map<String, String> getBody(Exception ex) {
        Map<String, String> body = new HashMap();
        if (ex instanceof SwismpBizException) {
            body.put("message", ex.getMessage());
            body.put("msg", HttpResponseParam.FAIL_MSG);
        } else if (ex instanceof IllegalArgumentException) {
            body.put("message", ex.getMessage());
            body.put("msg", HttpResponseParam.FAIL_MSG);
        } else {
            body.put("message", "系统出现未知异常，请联系管理员！");
            body.put("msg", HttpResponseParam.FAIL_MSG);
        }
        return body;
    }
}
