package com.sunwin.swismp.util.mobileMessage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
* @Description: 短信发送工具
* @Author: zc
* @CreateDate: 2019/2/28 11:33
* @Remark:
* @Version: 1.0
*/
public class MobileMessageSendUtil {

    //发送验证码的请求路径URL
    private static final String SERVER_URL= ReadPropertiesUtil.GetValueByKey("netease_im_sendcode");
    //发送带有参数的url
    private static final String SEND_MSG_SERVER_URL = ReadPropertiesUtil.GetValueByKey("netease_im_sendtemplate");
    //网易云信分配的账号，请替换你在管理后台应用下申请的Appkey
    private static final String APP_KEY = ReadPropertiesUtil.GetValueByKey("swbd_appKey");
    //网易云信分配的密钥，请替换你在管理后台应用下申请的appSecret
    private static final String APP_SECRET=ReadPropertiesUtil.GetValueByKey("swbd_appSecret");
    //随机数
    private static final String NONCE= String.valueOf(System.currentTimeMillis());

    //验证码长度，范围4～10，默认为4
    private static final String CODELEN="6";

    /**
     * 验证码接口
     * @param phone
     * @param templateid
     * @return
     * @throws IOException
     */
    public static NeteaseImResponse sendMsg(String phone, String templateid) throws IOException {
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost post = new HttpPost(SERVER_URL);

//      CheckSum的计算
        String curTime= String.valueOf((new Date().getTime()/1000L));
        String checkSum= CheckSumBuilder.getCheckSum(APP_SECRET,NONCE,curTime);

        //设置请求的header
        post.addHeader("AppKey",APP_KEY);
        post.addHeader("Nonce",NONCE);
        post.addHeader("CurTime",curTime);
        post.addHeader("CheckSum",checkSum);
        post.addHeader("Content-Type","application/x-www-form-urlencoded;charset=utf-8");

        //设置请求参数
        List<NameValuePair> nameValuePairs =new ArrayList<>();
        nameValuePairs.add(new BasicNameValuePair("mobile",phone));
        nameValuePairs.add(new BasicNameValuePair("templateid", ReadPropertiesUtil.GetValueByKey(templateid)));
        nameValuePairs.add(new BasicNameValuePair("codeLen", CODELEN));

        post.setEntity(new UrlEncodedFormEntity(nameValuePairs,"utf-8"));

        //执行请求
        HttpResponse response=httpclient.execute(post);
        String responseEntity= EntityUtils.toString(response.getEntity(),"utf-8");

        //获取发送状态码
        JSONObject jsonObject = JSON.parseObject(responseEntity);
        Integer code = jsonObject.getInteger("code");
        NeteaseImResponse neteaseImResponse = new NeteaseImResponse();
        if (code == 200){
            neteaseImResponse = JSON.parseObject(responseEntity, NeteaseImResponse.class);
        }else {
            neteaseImResponse.setCode(jsonObject.getInteger("code"));
            neteaseImResponse.setMsg(jsonObject.getString("msg"));
        }
        return neteaseImResponse;
    }

    /**
     * 带有参数的短信,可以批量发送多个对象
     * @param phones "['13888888888','13555555555']"
     * @param params 例如："['xxxx','xxxx']
     * @param templateid
     * @return
     * @throws IOException
     */
    public static NeteaseImResponse sendMsg(String phones, String params, String templateid) throws IOException {
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost post = new HttpPost(SEND_MSG_SERVER_URL);

//      CheckSum的计算
        String curTime= String.valueOf((new Date().getTime()/1000L));
        String checkSum= CheckSumBuilder.getCheckSum(APP_SECRET,NONCE,curTime);

        //设置请求的header
        post.addHeader("AppKey",APP_KEY);
        post.addHeader("Nonce",NONCE);
        post.addHeader("CurTime",curTime);
        post.addHeader("CheckSum",checkSum);
        post.addHeader("Content-Type","application/x-www-form-urlencoded;charset=utf-8");

        //设置请求参数
        List<NameValuePair> nameValuePairs =new ArrayList<>();
        nameValuePairs.add(new BasicNameValuePair("mobiles",phones));
        nameValuePairs.add(new BasicNameValuePair("templateid", ReadPropertiesUtil.GetValueByKey(templateid)));
        nameValuePairs.add(new BasicNameValuePair("codeLen", CODELEN));
        nameValuePairs.add(new BasicNameValuePair("params", params));

        post.setEntity(new UrlEncodedFormEntity(nameValuePairs,"utf-8"));

        //执行请求
        HttpResponse response=httpclient.execute(post);
        String responseEntity= EntityUtils.toString(response.getEntity(),"utf-8");

        //获取发送状态码
        return JSON.parseObject(responseEntity, NeteaseImResponse.class);
    }


    public static void main(String[] args) throws IOException {
        String[] params = new String[]{"8", "https://axhub.im/pro/02fc03a400783200/"};
        System.out.println(JSON.toJSONString(params));
        List<String> phones = new ArrayList<>();
        phones.add("17730212170");
//        phones.add("15656002738");
        System.out.println(JSON.toJSONString(phones));
        String params2 = "['8','https://axhub.im/pro']";
        NeteaseImResponse response = sendMsg(JSON.toJSONString(phones), params2,"bdiap_send_templateid");
        System.out.println(JSON.toJSONString(response));
    }



}
