package com.sunwin.swismp.config.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.env.Environment;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.PlatformTransactionManager;

import java.util.HashMap;
import java.util.Map;

/**
 * 数据源配置
 * es 是自动加载
 */
@Configuration
public class DataSourceConfig {

	@Autowired
	private Environment env;

	//
	@Bean(name = "dataSource3")
	public DruidDataSource dataSource3(){
		DruidDataSource dataSource = new DruidDataSource();
		dataSource.setUrl(env.getProperty("spring.datasource.postgresql.url"));
		dataSource.setUsername(env.getProperty("spring.datasource.postgresql.username"));
		dataSource.setPassword(env.getProperty("spring.datasource.postgresql.password"));
		dataSource.setDriverClassName(env.getProperty("spring.datasource.postgresql.driver-class-name"));
		return dataSource;
	}

	/**
	 * 动态数据源，通过AOP实现切换
	 * @return
	 */
	@Bean(name = "dynamicDataSource")
	@Primary
	public DynamicDataSource dynamicDataSource() {
		DynamicDataSource dynamicDataSource = new DynamicDataSource();
        //默认数据源，mysql
        dynamicDataSource.setDefaultTargetDataSource(dataSource3());
		//配置多数据源
		Map<Object, Object> map = new HashMap<>();
		//TODO:添加新的jdbc数据源类型
		map.put(DataSourceType.postgreDS,dataSource3());
//		map.put(DataSourceType.azkabanDS,dataSource4());
		dynamicDataSource.setTargetDataSources(map);
		return dynamicDataSource;
	}

	/**
	 * 配置事务管理器
	 * @return
	 */
	@Bean
	public PlatformTransactionManager transactionManager() {
		return new DataSourceTransactionManager(dynamicDataSource());
	}
}
