package com.sunwin.swismp.dao.sysfunction.mapper;

import com.sunwin.swismp.bean.sysfunction.SysRole;
import com.sunwin.swismp.dto.sysfunction.SysRoleDTO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

@Mapper
public interface SysRoleMapper {
    /**
     * 删除
     * @param roleId
     * @return
     */
    int deleteByPrimaryKey(Integer roleId);
    /**
     * @Description : 插入新增角色信息
     * @param record
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:36
    */
    int insert(SysRole record);

    /**
     * @Description : 根据主键修改角色内容
     * @param record
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:37
    */
    int updateByPrimaryKey(SysRole record);
    /**
     * @Description : 根据名称查询
     * @param roleName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:35
    */
    SysRole selectByRoleName(@Param(value = "roleName") String roleName);
    /**
     * @Description : 根据编号查询
     * @param roleCode
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:35
    */
    SysRole selectByRoleCode(@Param(value = "roleCode") String roleCode);
    /**
     * @Description : 根据角色名称查询 是否已有角色存在
     * @param roleName
     * @param roleId
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:38
    */
    SysRole selectByRoleNameOutSelf(@Param(value = "roleName") String roleName, @Param(value = "roleId") Integer roleId);
    /**
     * @Description : 根据编号查询是否已有角色存在
     * @param roleCode
     * @param roleId
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:43
    */
    SysRole selectByRoleCodeOutSelf(@Param(value = "roleCode") String roleCode, @Param(value = "roleId") Integer roleId);

    List<SysRole> selectAll();
    /**
     *  分页-条件查询
     * @param roleName 名称
     * @param roleCode 编号
     * @param isEnable 是否可用
     * @param offset 位移
     * @param size 数目
     * @return
     */
    List<SysRoleDTO> selectPage(@Param(value = "roleName") String roleName,
                                @Param(value = "roleCode") String roleCode,
                                @Param(value = "isEnable") Integer isEnable,
                                @Param(value = "keywords") String keywords,
                                @Param(value = "offset") Integer offset,
                                @Param(value = "size") Integer size);
    Integer selectPageCount(@Param(value = "roleName") String roleName,
                            @Param(value = "roleCode") String roleCode,
                            @Param(value = "isEnable") Integer isEnable,
                            @Param(value = "keywords") String keywords);
    /**
     * @Description : 检查角色名 是否 与 已有 角色名 部门 人名冲突
     * @param roleName
     * @param roleId
     * @Return :
     * @Author : yangdong
     * @Date : 2020/5/28 15:36
    */
    List<Map> checkRoleName(@Param("roleName") String roleName, @Param("roleId") Integer roleId);

    List<SysRole> selectExcludeRoleCode(@Param(value = "roleCode") String excludeRoleCode);

    /**
     * 查询角色用户数
     * @param roleId
     * @return
     */
    Integer selectRoleUserNum(@Param(value = "roleId") Integer roleId);
}