package com.sunwin.swismp.dao.sysfunction.mapper;

import com.sunwin.swismp.bean.sysfunction.SysUser;
import com.sunwin.swismp.dto.sysfunction.SysUserDTO;
import com.sunwin.swismp.dto.sysfunction.SysUserManage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

@Mapper
public interface SysUserMapper {

    /**
     * 添加用户
     * @param record
     * @return
     */
    int insert(SysUser record);
    /**
     * @Description : 根据id查询用户内容
     * @param userId
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:46
     */
    SysUser selectByPrimaryKey(Integer userId);
    /**
     * @Description : 更新用户信息
     * @param record
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:53
    */
    int updateByPrimaryKey(SysUser record);
    /**
     * @Description : 根据电话查询用户内容
     * @param telephone
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:47
     */
    SysUser selectByTelephone(@Param("telephone") String telephone);
    /**
     * @Description : 更新电话号码
     * @param userId
     * @param telephone
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:54
    */
    int updateTelephoneByUserId(@Param("userId") Integer userId, @Param("telephone") String telephone);
    /**
     * @Description : 更新用户邮箱
     * @param userId
     * @param email
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:55
    */
    int updateEmailByUserId(@Param("userId") Integer userId, @Param("email") String email);
    /**
     * @Description : 验证用户是否存在
     * @param userName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:51
     * @return
    */
    List<SysUserDTO> selectByUserName(@Param("userName") String userName);

    SysUser selectByUserNameWithRoleCode(@Param("userName") String userName);

    /**
     * @Description : 查找重合的部门 权限 用户名称
     * @param userName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/5/22 9:33
    */
    List<SysUser> selectUserRoleOrgByUserName(@Param("userName") String userName);
    /**
     * @Description : 更新用户密码
     * @param userId
     * @param userPwd
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:53
    */
    int updatePwd(@Param("userId") Integer userId, @Param("userPwd") String userPwd);
    /**
     * 注销,暂停用户
     * @param userId
     * @return
     */
    int logoff(@Param("userId") Integer userId);
    /**
     * 启用用户
     * @param userId
     * @param enabled
     * @return
     */
    int enableUser(@Param("userId") Integer userId, @Param("enabled") Integer enabled);
    /**
     * @Description : 删除用户内容 根据主键
     * @param userId
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:55
    */
    int deleteByPrimaryKey(@Param("userId") Integer userId);

    /**
     * 模糊匹配userName, phone, email三个字段
     * @return
     */
    List<SysUserManage> selectAllUserManage(@Param("param") String param, @Param("pageNo") Integer pageNo,
                                            @Param("pageSize") Integer pageSize, @Param("orgId") Integer orgId);
    /**
     * 查询所有系统用户管理信息
     * @return
     */
    SysUserManage selectUserManageById(@Param("userId") Integer userId);


    /**
     * 多参数模糊查询
     * @param userName
     * @param phone
     * @param email
     * @return
     */
    List<SysUserManage> selectUserManageByParams(@Param("userName") String userName, @Param("phone") String phone,
                                                 @Param("email") String email, @Param("pageNo") Integer pageNo,
                                                 @Param("pageSize") Integer pageSize, @Param("orgId") Integer orgId);
    /**
     * @Description : 根据用户姓名 手机号  邮箱 返回用户编号
     * @param userName
     * @param phone
     * @param email
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:16
     */
    Integer selectUserManageByParamsCount(@Param("userName") String userName, @Param("phone") String phone,
                                          @Param("email") String email, @Param("orgId") Integer orgId);
    /**
     * @Description : 查询用户所有信息
     * @param param
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:17
     */
    Integer selectAllUserManageCount(@Param("param") String param, @Param("orgId") Integer orgId);

    /**
     * 查询所有的用户id 信息
     * @return
     */
    List<SysUserManage> queryAllUserIdName();
    List<SysUserManage> queryOrgUserRelation();
    List<SysUserManage> queryUserRoleRelation();

    List<SysUser> selectRoleOrgByUserName(@Param("userName") String userName, @Param("userId") Integer userId);
    /**
     * @Description : 查找用户权限
     * @param userName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/6/18 8:32
    */
    Map queryUserEnabled(@Param("userName") String userName);

    /**
     * 查询系统所有用户
     * @return
     */
    List<SysUserManage> queryUserManger();

    List<SysUserDTO> querySysUserPage(@Param("orgType") String orgType, @Param("orgId") Integer orgId,
                                      @Param("roleId") String roleId, @Param("realName") String realName,
                                      @Param("isEnabled") String isEnabled, @Param("userName") String userName,
                                      @Param("beginUpdateTime") String beginUpdateTime,
                                      @Param("endUpdateTime") String endUpdateTime, @Param("pageNum") Integer pageNum,
                                      @Param("pageSize") Integer pageSize);

    Long querySysUserPageCount(@Param("orgType") String orgType, @Param("orgId") Integer orgId,
                               @Param("roleId") String roleId, @Param("realName") String realName,
                               @Param("isEnabled") String isEnabled, @Param("userName") String userName,
                               @Param("beginUpdateTime") String beginUpdateTime,
                               @Param("endUpdateTime") String endUpdateTime);

    SysUserDTO queryUserById(@Param("userId") Integer userId);

    List<SysUserDTO> querySysUserByName(@Param("userName") String userName);

    String queryLastLoginTimeByUserName(@Param("userName") String userName);
}