package com.sunwin.swismp.service.sysfunction.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.swismp.bean.TreeNode;
import com.sunwin.swismp.bean.sysfunction.SysMenu;
import com.sunwin.swismp.config.annotation.DS;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.dao.sysfunction.mapper.SysMenuMapper;
import com.sunwin.swismp.config.datasource.DataSourceType;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysMenuDTO;
import com.sunwin.swismp.dto.sysfunction.SysMenuResult;
import com.sunwin.swismp.service.sysfunction.SysMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 菜单信息服务
 */
@Service
public class SysMenuServiceImpl implements SysMenuService {

    @Autowired
    private SysMenuMapper sysMenuMapper;

    /**
     * 查询角色能看到的菜单
     * @param roleId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<SysMenuResult> selectByRoleId(Integer roleId) {
        return constructMenus(sysMenuMapper.selectByRoleId(roleId));
    }

    /**
     * 用户id对应的菜单
     * @param userId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<SysMenuResult> selectByUserId(Integer userId) {
        List<SysMenu> menus = sysMenuMapper.selectByUserId(userId);
        return constructMenus(menus);
    }


    /**
     * 构造多级菜单
     * @param menus
     * @return
     */
    public List<SysMenuResult> constructMenus(List<SysMenu> menus) {
        List<SysMenuResult> sysMenuResults = new ArrayList<>();
        for (SysMenu sysMenu : menus) {
            sysMenuResults.add(new SysMenuResult(sysMenu));
        }

        for (SysMenuResult sysMenuResult1 : sysMenuResults) {
            for (SysMenuResult sysMenuResult2 : sysMenuResults) {
                if (sysMenuResult1.getMenuId().equals(sysMenuResult2.getParentMenuId())) {
                    sysMenuResult1.addChildMenu(sysMenuResult2);
                }
            }
        }

        List<SysMenuResult> results = new ArrayList<>();
        for (SysMenuResult sysMenuResult : sysMenuResults) {
            if (sysMenuResult.getParentMenuId() == null) {
                results.add(sysMenuResult);
            }
        }
        return results;
    }

    /**
     * 菜单明细
     * @param menuId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysMenuDTO selectMenuById(Integer menuId){
        SysMenuDTO sysMenu = sysMenuMapper.selectMenuById(menuId);
        return sysMenu==null?new SysMenuDTO():sysMenu;
    }

    /**
     * 查询次级菜单列表
     * @param menuId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public PageResult queryMenuSecondary(Integer menuId,
                                         Integer resourceType, Integer isEnable, String keywords,
                                         Integer pageNum, Integer pageSize){
        Integer offset = (pageNum - 1) * pageSize;
        List<SysMenuDTO> sysMenuDTOS = sysMenuMapper.queryMenuSecondary(menuId,
                resourceType, isEnable, keywords,
                offset, pageSize);
        Long count = sysMenuMapper.queryMenuSecondaryCount(menuId,
                resourceType, isEnable, keywords);
        PageResult pageResult = new PageResult();
        pageResult.setData(sysMenuDTOS);
        pageResult.setCount(count);
        return pageResult;
    }

    /**
     * 保存系统资源信息（新增、修改）
     * @param sysMenu
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public HttpResult menuSave(SysMenu sysMenu){
        HttpResult httpResult = null;
        int i;
        Integer menuId = sysMenu.getMenuId();
        //为空判断
        if(sysMenu.getMenuName()==null || "".equals(sysMenu.getMenuName())){
            httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,"资源名称不可为空","资源名称不可为空");
            return httpResult;
        }
        if(sysMenu.getResourceType()==null){
            httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,"资源类型不可为空","资源类型不可为空");
            return httpResult;
        }
        String  msg =null;
        Integer msgCode = 1;
        Long count =0L;
        //判断 新增 或 修改
        if (menuId == null){
            count = sysMenuMapper.queryMenuCount(null, sysMenu.getMenuName(),null, null, null);
            if (count>0) {
                i = 0;
                msg = HttpResponseParam.FAIL_ADD_DUPLICATTED_MSG;
                msgCode = HttpResponseParam.RECORD_THE_EXISTING_CODE;
            }else {
                i = sysMenuMapper.menuInsert(sysMenu);
                msg = HttpResponseParam.SUCCESS_INSERT_MSG;
                msgCode = HttpResponseParam.SUCCESS_CODE;
            }

        } else {
            count = sysMenuMapper.queryMenuOutSelfCount(menuId, sysMenu.getMenuName());
            if (count>0) {
                i = 0;
                msg = HttpResponseParam.FAIL_UPDATE_DUPLICATTED_MSG;
                msgCode = HttpResponseParam.RECORD_THE_EXISTING_CODE;
            }else {
                i = sysMenuMapper.menuUpdateByPrimaryKey(sysMenu);
                msg = HttpResponseParam.SUCCESS_UPDATE_MSG;
                msgCode = HttpResponseParam.SUCCESS_CODE;
            }
        }
        if (i == 1){
            httpResult = new HttpResult(msgCode,msg,"保存成功");
        }else{
            httpResult = new HttpResult(msgCode,msg,"保存失败");
        }
        return httpResult;
    }

    /**
     * 修改启用状态
     * @param sysMenu
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer updateEnabled(SysMenu sysMenu){
        return sysMenuMapper.menuUpdateIsEnableByPrimaryKey(sysMenu);
    }

    /**
     * @Description : 删除资源
     * @Return :
     */
    @Override
    public Integer deleteMenu(Integer menuId){
        Long count = sysMenuMapper.queryMenuSecondaryCount(menuId, null, null, null);
        if (count>0L){
            return 2;
        }
        return sysMenuMapper.deleteMenu(menuId);
    }

    /**
     * @Description : 系统资源菜单树
     * @Return :
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<TreeNode> sysMenuTree(){
        List<SysMenuDTO> sysMenuDTOS = sysMenuMapper.querySysMenuTree();
        JSONArray jsonResult = treeRecursionDataList(sysMenuDTOS, 0);
        List<SysMenuDTO> sysOrgTreeList = jsonResult.toJavaList(SysMenuDTO.class);
        List<TreeNode> rootTree = new ArrayList<>();
        TreeNode root = new TreeNode(0, null, "智慧城轨大数据管理平台", true, false,
                false, true, 1);
        rootTree.add(root);
        if (CollectionUtils.isEmpty(sysMenuDTOS)) {
            return rootTree;
        }
        root.setChildren(treeNodes(sysOrgTreeList));
        return rootTree;
    }

    /**
     * @param sysMenuTreeList
     * @return
     */
    private List<TreeNode> treeNodes(List<SysMenuDTO> sysMenuTreeList) {
        List<TreeNode> treeNodes = new ArrayList<>();
        for (SysMenuDTO sysMenuDTO : sysMenuTreeList) {
            TreeNode treeNode = new TreeNode(sysMenuDTO.getMenuId(), sysMenuDTO.getMenuId(), sysMenuDTO.getMenuName(),
                    true, false, false, true, sysMenuDTO.getMenuLevel());
            if (!CollectionUtils.isEmpty(sysMenuDTO.getChildList())) {
                treeNode.setChildren(treeNodes(sysMenuDTO.getChildList()));
            }
            treeNodes.add(treeNode);
        }

        return treeNodes;
    }


    /**
     * @param treeList
     * @param parentId
     * @return
     */
    public static JSONArray treeRecursionDataList(List<SysMenuDTO> treeList, int parentId) {
        JSONArray childMenu = new JSONArray();
        for (Object object : treeList) {
            JSONObject jsonMenu = JSONObject.parseObject(JSON.toJSONString(object));
            int menuId = jsonMenu.getIntValue("menuId");
            int pid = jsonMenu.getIntValue("parentMenuId");
            if (parentId == pid) {
                JSONArray c_node = treeRecursionDataList(treeList, menuId);
                jsonMenu.put("childList", c_node);
                childMenu.add(jsonMenu);
            }
        }
        return childMenu;
    }

}
