package com.sunwin.swismp.service.sysfunction.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.swismp.bean.TreeNode;
import com.sunwin.swismp.config.annotation.DS;
import com.sunwin.swismp.config.datasource.DataSourceType;
import com.sunwin.swismp.dao.sysfunction.mapper.SysOrgMapper;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysOrgDTO;
import com.sunwin.swismp.service.sysfunction.SysOrgService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Copyright (c) 12/20/2021 All Rights Reserved By SUNWIN
 * <p>
 * Package: com.sunwin.swismp.service.sysfunction.impl
 * FileName: SysOrgServiceImpl
 *
 * @author duanmin
 * @description
 * @created 2/11/2022 - 5:26 PM
 * @last Modified
 * @history
 */
@Service
public class SysOrgServiceImpl implements SysOrgService {
    @Autowired
    private SysOrgMapper sysOrgMapper;

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer addSysOrg(SysOrgDTO sysOrgDTO) {
        List<SysOrgDTO> sysOrgDTOList = sysOrgMapper.querySysOrgByParentAndName(sysOrgDTO.getParentOrgId(),
                sysOrgDTO.getOrgName());
        if (!CollectionUtils.isEmpty(sysOrgDTOList)) {
            return 2;
        }
        return sysOrgMapper.addSysOrg(sysOrgDTO);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer delSysOrg(Integer sysOrgId) {
        List<SysOrgDTO> sysOrgDTOList = sysOrgMapper.querySysOrgByParentAndName(sysOrgId, null);
        if (!CollectionUtils.isEmpty(sysOrgDTOList)) {
            return 2;
        }
        return sysOrgMapper.delSysOrg(sysOrgId);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer editSysOrg(SysOrgDTO sysOrgDTO) {
        List<SysOrgDTO> sysOrgDTOList = sysOrgMapper.querySysOrgByParentAndName(sysOrgDTO.getParentOrgId(),
                sysOrgDTO.getOrgName());
        SysOrgDTO sysOrgDTO1 = sysOrgMapper.viewSysOrg(sysOrgDTO.getOrgId().toString());
        if (!CollectionUtils.isEmpty(sysOrgDTOList) && !sysOrgDTO.getOrgName().equals(sysOrgDTO1.getOrgName())) {
            return 2;
        }
        return sysOrgMapper.editSysOrg(sysOrgDTO);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer enableSysOrg(Integer sysOrgId, Integer enabled) {
        return sysOrgMapper.enableSysOrg(sysOrgId, enabled);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public PageResult querySysOrgPage(String parentOrgId, String orgUserNum, String isEnabled, String keywords,
                                      String beginUpdateTime, String endUpdateTime, Integer pageNum, Integer pageSize) {
        List<SysOrgDTO> sysOrgDTOList = sysOrgMapper.querySysOrgPage(parentOrgId, orgUserNum, isEnabled, keywords,
                beginUpdateTime, endUpdateTime, (pageNum - 1) * pageSize, pageSize);
        Long count = sysOrgMapper.querySysOrgPageCount(parentOrgId, orgUserNum, isEnabled, keywords, beginUpdateTime,
                endUpdateTime);
        return new PageResult(count, sysOrgDTOList);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<TreeNode> sysOrgTree() {
        List<SysOrgDTO> sysOrgDTOS = sysOrgMapper.querySysOrgTree();
        JSONArray jsonResult = treeRecursionDataList(sysOrgDTOS, 0);
        List<SysOrgDTO> sysOrgTreeList = jsonResult.toJavaList(SysOrgDTO.class);
        List<TreeNode> rootTree = new ArrayList<>();
        TreeNode root = new TreeNode(0, null, "合肥轨道交通", true, false,
                false, true, 1);
        rootTree.add(root);
        if (CollectionUtils.isEmpty(sysOrgDTOS)) {
            return rootTree;
        }
        root.setChildren(treeNodes(sysOrgTreeList));
        return rootTree;
    }

    /**
     * @param sysOrgTreeList
     * @return
     */
    private List<TreeNode> treeNodes(List<SysOrgDTO> sysOrgTreeList) {
        List<TreeNode> treeNodes = new ArrayList<>();
        for (SysOrgDTO sysOrgDTO : sysOrgTreeList) {
            TreeNode treeNode = new TreeNode(sysOrgDTO.getOrgId(), sysOrgDTO.getOrgId(), sysOrgDTO.getOrgName(),
                    true, false, false, true, sysOrgDTO.getOrgLevel());
            if (!CollectionUtils.isEmpty(sysOrgDTO.getChildList())) {
                treeNode.setChildren(treeNodes(sysOrgDTO.getChildList()));
            }
            treeNodes.add(treeNode);
        }

        return treeNodes;
    }


    /**
     * @param treeList
     * @param parentId
     * @return
     */
    public static JSONArray treeRecursionDataList(List<SysOrgDTO> treeList, int parentId) {
        JSONArray childMenu = new JSONArray();
        for (Object object : treeList) {
            JSONObject jsonMenu = JSONObject.parseObject(JSON.toJSONString(object));
            int orgId = jsonMenu.getIntValue("orgId");
            int pid = jsonMenu.getIntValue("parentOrgId");
            if (parentId == pid) {
                JSONArray c_node = treeRecursionDataList(treeList, orgId);
                jsonMenu.put("childList", c_node);
                childMenu.add(jsonMenu);
            }
        }
        return childMenu;
    }


    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysOrgDTO viewSysOrg(String sysOrgId) {
        List<SysOrgDTO> sysOrgDTOList = sysOrgMapper.querySysOrgByParentAndName(Integer.valueOf(sysOrgId), null);
        SysOrgDTO sysOrgDTO = sysOrgMapper.viewSysOrg(sysOrgId);
        sysOrgDTO.setChildOrgCount(sysOrgDTOList.size());
        return sysOrgDTO;
    }

}
