package com.sunwin.swismp.service.sysfunction.impl;

import com.sunwin.swismp.bean.sysfunction.SysRole;
import com.sunwin.swismp.bean.sysfunction.SysRoleMenu;
import com.sunwin.swismp.config.annotation.DS;
import com.sunwin.swismp.config.datasource.DataSourceType;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.dao.sysfunction.mapper.SysRoleMapper;
import com.sunwin.swismp.dao.sysfunction.mapper.SysRoleMenuMapper;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.sysfunction.SysRoleDTO;
import com.sunwin.swismp.service.sysfunction.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 角色信息服务
 */
@Service
public class SysRoleServiceImpl implements SysRoleService {

    @Autowired
    private SysRoleMapper sysRoleMapper;

    @Autowired
    private SysRoleMenuMapper sysRoleMenuMapper;

    /**
     * 删除
     * @param roleId
     * @return
     */
    @Override
    @DS(DataSourceType.postgreDS)
    public int deleteRoleByRoleId(Integer roleId) {
        Integer i = sysRoleMapper.selectRoleUserNum(roleId);
        if (i>0) return 2;
        return sysRoleMapper.deleteByPrimaryKey(roleId);
    }

    /**
     * 查询所有角色
     * @return
     */
    @Override
    @DS(DataSourceType.postgreDS)
    public List<SysRole> selectAll() {
        return sysRoleMapper.selectAll();
    }


    /**
     *  分页-条件查询
     * @param roleName
     * @param roleCode
     * @param isEnable
     * @param page
     * @param size
     * @return
     */
    @Override
    @DS(DataSourceType.postgreDS)
    public List<SysRoleDTO> selectPage(String roleName, String roleCode, Integer isEnable, String keywords, Integer page, Integer size) {
        Integer offset = (page - 1) * size;
        return sysRoleMapper.selectPage(roleName,roleCode,isEnable,keywords,offset,size);
    }
    /**
     * @Description : 通过roleCode 来判断 是否 需要查询所有的role列表
     * @param roleCode
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/25 11:08
    */
    @Override
    @DS(DataSourceType.postgreDS)
    public List<SysRole> selectAllByRoleCode(String excludeRoleCode) {
        if(excludeRoleCode.equals("10001")){
            return sysRoleMapper.selectAll();
        } else {
            return sysRoleMapper.selectExcludeRoleCode(excludeRoleCode);
        }
    }

    /**
     * @Description : 查询分页角色信息
     * @param roleName
     * @param roleCode
     * @param isEnable
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:15
    */
    @Override
    @DS(DataSourceType.postgreDS)
    public Integer selectPageCount(String roleName,String roleCode,Integer isEnable,String keywords) {
        return sysRoleMapper.selectPageCount(roleName,roleCode,isEnable,keywords);
    }

    /**
     * 保存角色信息（新增、修改）
     * @param sysRole
     * @return
     */
    @Override
    @DS(DataSourceType.postgreDS)
    public HttpResult roleSave(SysRole sysRole){
        HttpResult httpResult = null;
        int i;
        Integer roleId = sysRole.getRoleId();
        //为空判断
        if(sysRole.getRoleName()==null || "".equals(sysRole.getRoleName())){
            httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,"角色名不可为空","角色名不可为空");
            return httpResult;
        }
       String  msg =null;
        //判断 新增 或 修改
        if (roleId == null){
            List<Map> nameMap = sysRoleMapper.checkRoleName(sysRole.getRoleName(),null);
            SysRole sysRole2 = sysRoleMapper.selectByRoleCode(sysRole.getRoleCode());
            if (nameMap != null && nameMap.size()>0){
                Map nameExisted = nameMap.get(0);
                switch (String.valueOf(nameExisted.get("type_existed"))){
                    case "role":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.ROLE_NAME_ERROR_MSG);
                        break;
                    case "user":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.USER_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                    case "org":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.ORG_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
              }
                return httpResult;
            }
            if(sysRole2!=null){
                httpResult = new HttpResult(HttpResponseParam.ROLE_CODE_ERROR_CODE,HttpResponseParam.ROLE_CODE_ERROR_MSG,HttpResponseParam.ROLE_CODE_ERROR_MSG);
                return httpResult;
            }
            sysRole.setIsEnable(0);
            sysRole.setCreateTime(new Date());
            sysRole.setUpdateTime(new Date());
            i = sysRoleMapper.insert(sysRole);
            //自动设置角色代码
            SysRole sysRoleNew = sysRoleMapper.selectByRoleName(sysRole.getRoleName());
            sysRoleNew.setRoleCode(String.format("%04d", sysRoleNew.getRoleId()));
            sysRoleMapper.updateByPrimaryKey(sysRoleNew);
            msg = HttpResponseParam.SUCCESS_INSERT_MSG;
        }else{
            SysRole sysRole3 = sysRoleMapper.selectByRoleNameOutSelf(sysRole.getRoleName(),roleId);
            SysRole sysRole4 = sysRoleMapper.selectByRoleCodeOutSelf(sysRole.getRoleCode(), roleId);
            if (sysRole3!=null){
                httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.ROLE_NAME_ERROR_MSG,HttpResponseParam.ROLE_NAME_ERROR_MSG);
                return httpResult;
            }
            if(sysRole4!=null){
                httpResult = new HttpResult(HttpResponseParam.ROLE_CODE_ERROR_CODE,HttpResponseParam.ROLE_CODE_ERROR_MSG,HttpResponseParam.ROLE_CODE_ERROR_MSG);
                return httpResult;
            }
            List<Map> nameMap = sysRoleMapper.checkRoleName(sysRole.getRoleName(),roleId);
            if (nameMap != null && nameMap.size()>0){
                Map nameExisted = nameMap.get(0);
                switch (String.valueOf(nameExisted.get("type_existed"))){
                    case "role":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.ROLE_NAME_ERROR_MSG);
                        break;
                    case "user":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.USER_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                    case "org":
                        httpResult = new HttpResult(HttpResponseParam.ROLE_NAME_ERROR_CODE,HttpResponseParam.ORG_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                }
                return httpResult;
            }
            sysRole.setUpdateTime(new Date());
            i = sysRoleMapper.updateByPrimaryKey(sysRole);
            msg = HttpResponseParam.SUCCESS_UPDATE_MSG;
        }
        if (i == 1){
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE,msg,"保存成功");
        }else{
            httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE,HttpResponseParam.SUCCESS_MSG,"保存失败");
        }
        return httpResult;
    }

    /**
     * 修改角色菜单对应关系
     * @param roleId
     * @param menuIds
     * @return
     */
    @Override
    //@Transactional
    @DS(DataSourceType.postgreDS)
    public boolean updateRoleMenu(Integer roleId,List<Integer> menuIds) {
        if (menuIds != null) {
            List<SysRoleMenu> sysRoleMenus = new ArrayList<>();
            for (Integer menuId : menuIds) {
                sysRoleMenus.add(new SysRoleMenu(roleId, menuId));
            }
            //先删除该角色对应的菜单
            sysRoleMenuMapper.deleteByRoleId(roleId);
            if (sysRoleMenus.size()>0){
                //插入新的对应关系
                sysRoleMenuMapper.batchInsert(sysRoleMenus);
            }
            return true;
        }
        return false;
    }

}
