package com.sunwin.swismp.service.sysfunction.impl;

import com.alibaba.dubbo.config.annotation.Service;
import com.sunwin.swismp.bean.sysfunction.SysUser;
import com.sunwin.swismp.config.annotation.DS;
import com.sunwin.swismp.config.datasource.DataSourceType;
import com.sunwin.swismp.constant.HttpResponseParam;
import com.sunwin.swismp.dao.sysfunction.mapper.SysUserMapper;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.*;
import com.sunwin.swismp.service.sysfunction.SysMenuService;
import com.sunwin.swismp.service.sysfunction.SysUserService;
import com.sunwin.swismp.util.Md5Util;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;
import java.util.regex.Pattern;

/**
 * 用户信息服务
 */
@Service
public class SysUserServiceImpl implements SysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysMenuService sysMenuService;
    @Autowired
    public RedisTemplate<String, String> redisTemplate;


    private DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    private ReentrantLock lock = new ReentrantLock();
    private String checkIpFailCount = "swismp_failCount-";
    private String checkIpExpireTime = "swismp_failExpireTime-";


    /**
     * 验证码存储到redis
     *
     * @param key
     * @param value
     * @param codeTimeOut 验证码过期时间，分钟
     */
    @Override
    public void setCodeRedis(String key, String value, Integer codeTimeOut) {
        redisTemplate.opsForValue().set(key, value, codeTimeOut, TimeUnit.MINUTES);
    }

    /**
     * 根据用户名查询用户内容
     *
     * @param userName add by fengyong
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysUser queryUserByName(String userName) {
        if (userName != null && !userName.equals("")) {
            return sysUserMapper.selectByUserName(userName).get(0);
        }
        return null;
    }

    /**
     * @param userId
     * @Description : 根据id查询用户内容
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:46
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysUser queryUserById(Integer userId) {
        return sysUserMapper.selectByPrimaryKey(userId);
    }

    /**
     * @param telephone
     * @Description : 根据电话查询用户内容
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/10 9:47
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysUser queryUserByTelephone(String telephone) {
        String pattern = "1[0-9]{10}"; //1开头，11位手机号
        if (telephone != null && !telephone.equals("") && Pattern.matches(pattern, telephone)) {
            return sysUserMapper.selectByTelephone(telephone);
        }
        return null;
    }

    /**
     * 添加用户
     *
     * @param user
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public HttpResult addUser(SysUser user) {
        HttpResult httpResult = null;
        String userName = user.getUserName();
        String passwd = user.getUserPwd();
        if (userName != null && !userName.trim().equals("") && passwd != null && !passwd.trim().equals("")) {
            //判断更新的用户名是否存在 或者是否与 部门 权限名重名
            List<SysUser> users = sysUserMapper.selectUserRoleOrgByUserName(userName);
            if (users != null && users.size() > 0) {
                SysUser userExisted = users.get(0);
                switch (userExisted.getRealName()) {
                    case "user":
                        httpResult = new HttpResult(HttpResponseParam.NAME_EXISTED_CODE, HttpResponseParam.NAME_EXISTED_MSG);
                        break;
                    case "role":
                        httpResult = new HttpResult(HttpResponseParam.NAME_EXISTED_CODE, HttpResponseParam.ROLE_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                    case "org":
                        httpResult = new HttpResult(HttpResponseParam.NAME_EXISTED_CODE, HttpResponseParam.ORG_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                }
            } else {
                //插入数据
                sysUserMapper.insert(user);
                httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_INSERT_MSG);
            }
        } else {
            httpResult = new HttpResult(HttpResponseParam.NAME_PASSWD_ERROR_CODE, HttpResponseParam.NAME_PASSWD_ERROR_MSG);
        }
        return httpResult;
    }

    /**
     * @Description : 返回部门 用户  角色 桑吉图
     * @Return :
     * @Author : yangdong
     * @Date : 2020/5/21 17:23
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public Map queryUserOrgRoleView() {
        List<SysUserManage> orgUserRelations = sysUserMapper.queryOrgUserRelation();
        List<SysUserManage> userRoleRelations = sysUserMapper.queryUserRoleRelation();
        LinkedHashMap<String, Integer> users = new LinkedHashMap();
        LinkedHashMap<String, Integer> orgs = new LinkedHashMap();
        LinkedHashMap<String, Integer> roles = new LinkedHashMap();
        List<Link> links = new ArrayList<>();
        List<Node> nodes = new ArrayList<>();
        /**
         * 创建 部门节点 以及 部门和用户关系
         */
        if (orgUserRelations != null && orgUserRelations.size() > 0) {
            for (SysUserManage orgUserMem : orgUserRelations) {
                orgs.put(orgUserMem.getOrgName(), orgs.getOrDefault(orgUserMem.getOrgName(), 0) + 1);
                Link link = new Link(orgUserMem.getOrgName(), orgUserMem.getUserName(), 1);
                links.add(link);
            }
            for (Map.Entry<String, Integer> entry : orgs.entrySet()) {
                nodes.add(new Node(entry.getKey(), entry.getValue()));
            }
        }
        /**
         * 创建 用户节点  权限节点  以及用户与权限的关系
         */
        if (userRoleRelations != null && userRoleRelations.size() > 0) {
            for (SysUserManage userRoleMem : userRoleRelations) {
                users.put(userRoleMem.getUserName(), users.getOrDefault(userRoleMem.getUserName(), 0) + 1);
                roles.put(userRoleMem.getRoleName(), roles.getOrDefault(userRoleMem.getRoleName(), 0) + 1);
                Link link = new Link(userRoleMem.getUserName(), userRoleMem.getRoleName(), 1);
                links.add(link);
            }
            for (Map.Entry<String, Integer> entry : users.entrySet()) {
                nodes.add(new Node(entry.getKey(), entry.getValue()));
            }
            for (Map.Entry<String, Integer> entry : roles.entrySet()) {
                nodes.add(new Node(entry.getKey(), entry.getValue()));
            }
        }
        Map map = new LinkedHashMap();
        if (nodes != null && nodes.size() > 0 && links != null && links.size() > 0) {
            map.put("nodes", nodes);
            map.put("links", links);
        }
        return map;
    }

    /**
     * 登陆
     *
     * @param userName
     * @param userPwd
     * @param vCode    手机验证码
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public Map login(String userName, String userPwd, String vCode) {
        HttpResult httpResult = null;
        Map map = new HashMap();
        if (userName != null && !userName.trim().equals("") && userPwd != null && !userPwd.trim().equals("")) {
            SysUser sysUser = sysUserMapper.selectByUserNameWithRoleCode(userName);
            Map userEnabled = sysUserMapper.queryUserEnabled(userName);
            //部门是否有权限 注释掉在这些会有空指针的异常
            //Object orgEnabled = userEnabled.get("orgEnabled");
            //角色是否有权限
            //Object roleEnabled = userEnabled.get("roleEnabled");
            if (sysUser == null) { //用户不存在
                httpResult = new HttpResult(HttpResponseParam.LOGIN_USERNAME_CODE, HttpResponseParam.LOGIN_USERNAME_MSG);

            } else if (!Md5Util.processUser(userPwd, sysUser, Md5Util.TimeMd5Type.DATE)) { //密码是否一致
                httpResult = new HttpResult(HttpResponseParam.LOGIN_PASSWORD_CODE, HttpResponseParam.LOGIN_PASSWORD_MSG);

            } else if (sysUser.getIsEnable() == null || sysUser.getIsEnable() != 1) { //账户是否注销
                httpResult = new HttpResult(HttpResponseParam.LOGIN_ISENABLE_CODE, HttpResponseParam.LOGIN_ISENABLE_MSG);
            } else if (userEnabled == null || userEnabled.get("orgenabled") == null || Integer.valueOf(String.valueOf(userEnabled.get("orgenabled"))) != 1) {//部门是否有权限
                httpResult = new HttpResult(HttpResponseParam.ORG_ISENABLE_CODE, HttpResponseParam.ORG_ISENABLE_MSG);
            } else if (userEnabled == null || userEnabled.get("roleenabled") == null || Integer.valueOf(String.valueOf(userEnabled.get("roleenabled"))) != 1) {//角色是否有权限
                httpResult = new HttpResult(HttpResponseParam.ROLE_ISENABLE_CODE, HttpResponseParam.ROLE_ISENABLE_MSG);
                //TODO 验证码暂时不处理,暂时注释
//            } else if(!redisTemplate.hasKey("swurt_login_telephone_"+sysUser.getMobilePhone())){
//                //是否获取验证码 是否正确 add by fengyong
//                httpResult = new HttpResult(HttpResponseParam.VERIFICATION_CODE_GET_CODE,HttpResponseParam.VERIFICATION_CODE_GET_MSG);
//            } else if(vCode == null ||
//                    !redisTemplate.hasKey("swurt_login_telephone_"+sysUser.getMobilePhone()) ||
//                    !(redisTemplate.opsForValue().get("swurt_login_telephone_"+sysUser.getMobilePhone())).equals(vCode)){
//                //手机验证码 是否正确 add by fengyong
//                httpResult = new HttpResult(HttpResponseParam.VERIFICATION_CODE_ISENABLE_CODE,HttpResponseParam.VERIFICATION_CODE_ISENABLE_MSG);
            } else {
                List<SysMenuResult> list = sysMenuService.selectByUserId(sysUser.getUserId());
                LoginResult loginResult = new LoginResult(sysUser.getUserId(), list);
                httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_LOG_MSG, loginResult); //登录成功，返回菜单列表
                map.put("u_feature", sysUser.getUserId());
                map.put("org_feature", sysUser.getOrgId());
                map.put("role_feature", sysUser.getRoleCode());
                map.put("role_name", sysUser.getRoleName());
            }
        } else {
            httpResult = new HttpResult(HttpResponseParam.FAIL_CODE, HttpResponseParam.FAIL_FAIL_MSG);
        }
        map.put("result", httpResult);
        return map;
    }

    /**
     * @param username
     * @return
     * @Description : 记录用户连续非法登录次数
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/27 8:34
     */
    @Override
    public Map<String, String> updateFailCount(String username) {
        Map<String, String> countRemain = new HashMap<String, String>();
        Boolean countUsed = false;
        try {
            lock.lock();
            if (lock(username)) {
                countUsed = true;
            } else {
                Long size = redisTemplate.opsForList().size(checkIpFailCount + username);
                if (size >= 4) {
                    redisTemplate.opsForValue().set(checkIpExpireTime + username, String.valueOf(System.currentTimeMillis()), 30, TimeUnit.MINUTES);
                    redisTemplate.delete(checkIpFailCount + username);
                    countUsed = true;
                } else {
                    redisTemplate.opsForList().rightPush(checkIpFailCount + username, "1");
                    //连续登录检测只设置一段分钟
                    if (size == 0) {
                        redisTemplate.expire(checkIpFailCount + username, 30, TimeUnit.MINUTES);
                    }
                    countRemain.put("countRemain", String.valueOf(4 - size));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
        }
        countRemain.put("locakedStart", countUsed.toString());
        return countRemain;
    }

    /**
     * @param username
     * @Description : 检查用户名 是否是锁定状态
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/27 8:33
     */
    @Override
    public Boolean lock(String username) {
        Boolean countUsed = false;
        String expire = redisTemplate.opsForValue().get(checkIpExpireTime + username);
        if (expire != null) {
            countUsed = true;
        }
        return countUsed;
    }

    /**
     * @param userName
     * @Description : 对用户锁定状态解锁
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/27 8:33
     */
    @Override
    public void unlock(String userName) {
        redisTemplate.delete(checkIpFailCount + userName);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer addSysUser(SysUserDTO sysUserDTO) {
        List<SysUserDTO> sysUserDTOList = sysUserMapper.querySysUserByName(sysUserDTO.getUserName());
        String loginTimeByUserName = sysUserMapper.queryLastLoginTimeByUserName(sysUserDTO.getUserName());
        if (!CollectionUtils.isEmpty(sysUserDTOList) || !StringUtils.isEmpty(loginTimeByUserName)) {
            return 2;
        }
        return sysUserMapper.insert(sysUserDTO);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public PageResult querySysUserPage(Integer orgId, String orgType, String roleId, String realName, String isEnabled,
                                       String userName, String beginUpdateTime, String endUpdateTime, Integer pageNum,
                                       Integer pageSize) {
        List<SysUserDTO> sysUserDTOList = sysUserMapper.querySysUserPage(orgType, orgId, roleId, realName, isEnabled,
                userName, beginUpdateTime, endUpdateTime, (pageNum - 1) * pageSize, pageSize);
        Long count = sysUserMapper.querySysUserPageCount(orgType, orgId, roleId, realName, isEnabled, userName,
                beginUpdateTime, endUpdateTime);
        return new PageResult(count, sysUserDTOList);
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysUserDTO userView(Integer userId) {
        SysUserDTO sysUserDTO = sysUserMapper.queryUserById(userId);
        String lastLoginTime = sysUserMapper.queryLastLoginTimeByUserName(sysUserDTO.getUserName());
        sysUserDTO.setLastLoginTime(lastLoginTime);
        return sysUserDTO;
    }

    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer editUser(SysUser user) {
        SysUserDTO sysUserDTO = sysUserMapper.queryUserById(user.getUserId());
        List<SysUserDTO> sysUserDTOList = sysUserMapper.querySysUserByName(user.getUserName());
        String loginTimeByUserName = sysUserMapper.queryLastLoginTimeByUserName(user.getUserName());
        //第一种情况是啥都没改，点击编辑按钮

        //第二种情况是改的用户id被之前删除过，有过操作日志，这种不可以修改
        if (!user.getUserName().equals(sysUserDTO.getUserName())
                && (!CollectionUtils.isEmpty(sysUserDTOList) || !StringUtils.isEmpty(loginTimeByUserName))) {
            return 2;
        }
        return sysUserMapper.updateByPrimaryKey(user);
    }

    /**
     * 更新用户信息
     *
     * @param user
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public HttpResult updateUser(SysUser user) {
        HttpResult httpResult = null;
        String userName = user.getUserName();
        Integer userId = user.getUserId();
        if (userName != null && !userName.equals("") && userId != null && userId > -1) {
            //判断更新的用户名 是否与 部门 权限名重名
            List<SysUser> users = sysUserMapper.selectRoleOrgByUserName(userName, userId);
            if (users != null && users.size() > 0) {
                SysUser userExisted = users.get(0);
                switch (userExisted.getRealName()) {
                    case "role":
                        httpResult = new HttpResult(HttpResponseParam.NAME_EXISTED_CODE, HttpResponseParam.ROLE_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                    case "org":
                        httpResult = new HttpResult(HttpResponseParam.NAME_EXISTED_CODE, HttpResponseParam.ORG_NAME_CONFLICT_WITH_NAME_MSG);
                        break;
                }
            } else {
                SysUser exitUser = sysUserMapper.selectByPrimaryKey(user.getUserId());
                if (exitUser != null) {
                    exitUser.setUserName(user.getUserName());
                    exitUser.setMobilePhone(user.getMobilePhone());
                    exitUser.setUserMail(user.getUserMail());
                    exitUser.setOrgId(user.getOrgId());
                    exitUser.setRoleId(user.getRoleId());
                    exitUser.setUpdateTime(new Date());
                    sysUserMapper.updateByPrimaryKey(exitUser);
                    httpResult = new HttpResult(HttpResponseParam.SUCCESS_CODE, HttpResponseParam.SUCCESS_UPDATE_MSG);
                } else {
                    httpResult = new HttpResult(HttpResponseParam.LOGIN_USERNAME_CODE, HttpResponseParam.LOGIN_USERNAME_MSG);
                }
            }
        } else {
            httpResult = new HttpResult(HttpResponseParam.NAME_PASSWD_ERROR_CODE, HttpResponseParam.NAME_PASSWD_ERROR_MSG);
        }
        return httpResult;
    }

    /**
     * 更新用户密码
     *
     * @param userId
     * @param newPwd
     * @param oldPwd
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public int updatePwd(Integer userId, String newPwd, String oldPwd) {
        SysUser sysUser = sysUserMapper.selectByPrimaryKey(userId);
        if (sysUser != null && Md5Util.processUserId(oldPwd, sysUser, Md5Util.TimeMd5Type.DATE)) { //如果原密码一致执行更新
            return sysUserMapper.updatePwd(userId, newPwd);
        }
        return 0;
    }

    /**
     * 重置用户密码
     *
     * @param userId
     * @param newPwd
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public int resetPwd(Integer userId, String newPwd) {
        SysUser sysUser = sysUserMapper.selectByPrimaryKey(userId);
        if (sysUser != null) { //如果原密码一致执行更新
            return sysUserMapper.updatePwd(userId, newPwd);
        }
        return 0;
    }

    /**
     * 用户修改电话号码
     *
     * @param newTelephone
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public boolean updateTelephone(Integer userId, String newTelephone) {
        //TODO: 暂时不用验证码
//     public boolean updateTelephone(Integer userId, String newTelephone, String newCode) {
//        String code2 = redisTemplate.opsForValue().get("telephone:" + newTelephone);
//        if (!code2.equals(newCode)) {
//            return false;
//        }
        SysUser sysUser = sysUserMapper.selectByTelephone(newTelephone); //验证新号码是否已有其他用户注册
        if (sysUser != null && sysUser.getUserId() != userId) {
            return false;
        }
        int i = sysUserMapper.updateTelephoneByUserId(userId, newTelephone);
        return i > 0;
    }

    /**
     * 用户修改邮箱
     *
     * @param userId
     * @param email
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public boolean updateEmail(Integer userId, String email) {
        int i = sysUserMapper.updateEmailByUserId(userId, email);
        return i == 1;
    }

    /**
     * 删除用户
     *
     * @param userId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public int deleteUser(Integer userId) {
        return sysUserMapper.deleteByPrimaryKey(userId);
    }

    /**
     * 注销,暂停用户
     *
     * @param userId
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public int logoff(Integer userId) {
        return sysUserMapper.logoff(userId);
    }

    /**
     * 启用用户
     *
     * @param userId
     * @param enabled
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public int enableUser(Integer userId, Integer enabled) {
        return sysUserMapper.enableUser(userId, enabled);
    }

    /**
     * 查询所有系统用户管理信息
     *
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<SysUserManage> selectAllUserManage(String param, Integer pageNo, Integer pageSize, Integer orgId) {
        return sysUserMapper.selectAllUserManage(param, pageNo, pageSize, orgId);
    }

    /**
     * 查询指定用户管理信息
     *
     * @return
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public SysUserManage selectUserManageById(Integer userId) {
        return sysUserMapper.selectUserManageById(userId);
    }

    /**
     * @param userName 名字
     * @param phone    电话
     * @param email    邮箱
     * @param pageNo   第几页
     * @param pageSize 每页条数
     * @Description : 多条件查询用户信息
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/9 17:14
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public List<SysUserManage> selectUserManageByParams(String userName, String phone, String email, Integer pageNo, Integer pageSize, Integer orgId) {
        return sysUserMapper.selectUserManageByParams(userName, phone, email, pageNo, pageSize, orgId);
    }

    /**
     * @param userName
     * @param phone
     * @param email
     * @Description : 根据用户姓名 手机号  邮箱 返回用户编号
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:16
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer selectUserManageByParamsCount(String userName, String phone, String email, Integer orgId) {
        return sysUserMapper.selectUserManageByParamsCount(userName, phone, email, orgId);
    }

    /**
     * @param param
     * @Description : 查询用户所有信息
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:17
     */
    @Override
    @DS(value = DataSourceType.postgreDS)
    public Integer selectAllUserManageCount(String param, Integer orgId) {
        return sysUserMapper.selectAllUserManageCount(param, orgId);
    }
}
