package com.sunwin.swismp.service.sysfunction;


import com.sunwin.swismp.bean.sysfunction.SysUser;
import com.sunwin.swismp.dto.HttpResult;
import com.sunwin.swismp.dto.PageResult;
import com.sunwin.swismp.dto.sysfunction.SysUserDTO;
import com.sunwin.swismp.dto.sysfunction.SysUserManage;

import java.util.List;
import java.util.Map;

public interface SysUserService {
    /**
     * 验证码存储到redis
     * @param key
     * @param value
     * @param codeTimeOut  验证码过期时间，分钟
     */
     void setCodeRedis(String key, String value, Integer codeTimeOut);

    /**
     * 根据用户名查询用户内容
     * @param userName
     * add by fengyong
     * @return
     */
    SysUser queryUserByName(String userName);
    SysUser queryUserById(Integer userId);
    SysUser queryUserByTelephone(String telephone);
    /**
     * 添加用户
     * @param user
     * @return
     */
    HttpResult addUser(SysUser user);

    Map queryUserOrgRoleView();

    /**
     * 登陆
     * @param userName
     * @param userPwd
     * @return
     */
    Map login(String userName, String userPwd, String vCode);
    /**
     * 更新用户信息
     * @param user
     * @return
     */
    HttpResult updateUser(SysUser user);
    /**
     * 更新用户密码
     * @param userId
     * @param newPwd
     * @param oldPwd
     * @return
     */
    int updatePwd(Integer userId, String newPwd, String oldPwd);
    /**
     * 删除用户
     * @param userId
     * @return
     */
    int deleteUser(Integer userId);
    /**
     * 注销,暂停用户
     * @param userId
     * @return
     */
    int logoff(Integer userId);
    /**
     * 启用用户
     * @param userId
     * @param enabled
     * @return
     */
    int enableUser(Integer userId, Integer enabled);
    /**
     * 查询所有系统用户管理信息
     * @return
     */
    List<SysUserManage> selectAllUserManage(String param, Integer pageNo, Integer pageSize, Integer orgId);
    /**
     * 查询指定用户管理信息
     * @return
     */
    SysUserManage selectUserManageById(Integer userId);
    /**
     * @Description : 多条件查询用户信息
     * @param userName  名字
     * @param phone  电话
     * @param email 邮箱
     * @param pageNo 第几页
     * @param pageSize 每页条数
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/9 17:14
     */
    List<SysUserManage> selectUserManageByParams(String userName, String phone, String email, Integer pageNo, Integer pageSize, Integer orgId);
    /**
     * @Description : 根据用户姓名 手机号  邮箱 返回用户编号
     * @param userName
     * @param phone
     * @param email
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:16
     */
    Integer selectUserManageByParamsCount(String userName, String phone, String email, Integer orgId);
    /**
     * @Description : 查询用户所有信息数
     * @param param
     * @Return :
     * @Author : yangdong
     * @Date : 2020/2/28 15:17
     */
    Integer selectAllUserManageCount(String param, Integer orgId);
    /**
     * 重置用户密码
     * @param userId
     * @param newPwd
     * @return
     */
    int resetPwd(Integer userId, String newPwd);
    /**
     * 用户修改电话号码
     *
     * @param newTelephone
     * @param userId
     * @return
     */
    boolean updateTelephone(Integer userId, String newTelephone);
    //TODO: 暂时不用验证码
    //boolean updateTelephone(Integer userId, String newTelephone, String newCode);
    /**
     * 用户修改邮箱
     *
     * @param userId
     * @param email
     * @return
     */
    boolean updateEmail(Integer userId, String email);

    /**
     * @Description : 更新该ip的连续登录失败次数   如果 连续超过5次 则直接锁定
     * @param userName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/25 16:08
     * @return
    */
    Map<String,String> updateFailCount(String userName);
    /**
     * @Description : 检查是否是锁定状态
     * @param userName
     * @Return :
     * @Author : yangdong
     * @Date : 2020/9/25 16:09
    */
    Boolean lock(String userName);

    void unlock(String userName);

    Integer addSysUser(SysUserDTO sysUserDTO);

    PageResult querySysUserPage(Integer orgId, String orgType, String roleId, String realName, String isEnabled,
                                String userName, String beginUpdateTime, String endUpdateTime, Integer pageNum,
                                Integer pageSize);

    SysUserDTO userView(Integer userId);

    Integer editUser(SysUser user);
}
