package com.sunwin.swismp.util;

import org.apache.logging.log4j.core.util.CronExpression;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName: CronUtil
 * @Description: Cron表达式工具类 目前支持三种常用的cron表达式 1.每天的某个时间点执行 例:12 12 12 * *
 *               ?表示每天12时12分12秒执行 2.每周的哪几天执行 例:12 12 12 ? * 1,2,3表示每周的周1周2周3
 *               ,12时12分12秒执行 3.每月的哪几天执行 例:12 12 12 1,21,13 * ?表示每月的1号21号13号
 *               12时12分12秒执行
 * @author
 * @date
 *
 */
public class CronUtil {
    /**
     * 通过输入指定日期时间生成cron表达式
     * @param date
     * @return cron表达式
     */
    public static String getCron(Date date){
        String dateFormat="ss mm HH dd MM ? yyyy";
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        String formatTimeStr = null;
        if (date != null) {
            formatTimeStr = sdf.format(date);
        }
        return formatTimeStr;
    }

    /**
     * cron 获取去执行时间计划（取前N个）
     * @param cronExpression  cron表达式
     * @param num 获取个数
     * @return
     */
    public static List<String> parser(String cronExpression,Integer num) {
        List<String> result = new ArrayList<String>();
        if (cronExpression == null || cronExpression.length() < 1) {
            return result;
        } else {
            CronExpression exp = null;
            try {
                exp = new CronExpression(cronExpression);
            } catch (ParseException e) {
                e.printStackTrace();
                return result;
            }
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date dd = new Date();
            dd = exp.getNextValidTimeAfter(dd);
            //取前十个
            for (int i=0;i<num;i++){
                result.add(sdf.format(dd));
                dd = exp.getNextValidTimeAfter(dd);
            }
            exp = null;
        }
        return result;
    }

    static final String xin="*";
    static final String wenhao="?";
    static final String dao="-";
    static final String mei="/";
    static final String huo=",";

    public static String descCorn(String cronExp) {
        String[] tmpCorns = cronExp.split(" ");
        StringBuffer sBuffer = new StringBuffer();
        if (tmpCorns.length != 6) {
            throw new RuntimeException("请补全表达式,必须标准的cron表达式才能解析");
        }
        // 解析月
        descMonth(tmpCorns[4], sBuffer);
        // 解析周
        descWeek(tmpCorns[5], sBuffer);

        // 解析日
        descDay(tmpCorns[3], sBuffer);

        // 解析时
        descHour(tmpCorns[2], sBuffer);

        // 解析分
        descMintue(tmpCorns[1], sBuffer);

        // 解析秒
        //descSecond(tmpCorns[0], sBuffer);
        sBuffer.append(" 执行");
        return sBuffer.toString();

    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:01:09
     */
    private static void descSecond(String s, StringBuffer sBuffer) {
        String danwei="秒";
        desc(s, sBuffer, danwei);
    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @param danwei
     * @author Norton Lai
     * @created 2019-2-27 下午5:16:19
     */
    private static void desc(String s, StringBuffer sBuffer, String danwei) {
        if (s.equals("1/1")) {
            s="*";
        }
        if (s.equals("0/0")) {
            s="0";
        }
        if (xin.equals(s)) {
            sBuffer.append("每"+danwei);
            return;
        }
        if (wenhao.equals(s)) {
            return ;
        }
        if (s.contains(huo)) {
            String[] arr = s.split(huo);
            for (int i = 0; i < arr.length; i++) {
                if (arr[i].length()!=0) {
                    sBuffer.append("第"+arr[i]+danwei+"和");
                }
            }
            sBuffer.deleteCharAt(sBuffer.length()-1);
            sBuffer.append("的");
            return;
        }
        if (s.contains(dao)) {
            String[] arr = s.split(dao);
            if (arr.length!=2) {
                throw new RuntimeException("表达式错误"+s);
            }
            sBuffer.append("从第"+arr[0]+danwei+"到第"+arr[1]+danwei+"每"+danwei);
            sBuffer.append("的");
            return;
        }

        if (s.contains(mei)) {
            String[] arr = s.split(mei);
            if (arr.length!=2) {
                throw new RuntimeException("表达式错误"+s);
            }
            if (arr[0].equals(arr[1])||arr[0].equals("0")) {
                sBuffer.append("每"+arr[1]+danwei);
            }else {
                sBuffer.append("每"+arr[1]+danwei+"的第"+arr[0]+danwei);
            }
            return;
        }
        sBuffer.append("第"+s+danwei);
    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:01:00
     */
    private static void descMintue(String s, StringBuffer sBuffer) {
        desc(s, sBuffer, "分钟");
    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:00:50
     */
    private static void descHour(String s, StringBuffer sBuffer) {
        desc(s, sBuffer, "小时");
    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:00:39
     */
    private static void descDay(String s, StringBuffer sBuffer) {
        desc(s, sBuffer, "天");
    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:00:30
     */
    private static void descWeek(String s, StringBuffer sBuffer) {
        //不解释 太麻烦
    }
//    private static String turnWeek(String week){
//        switch (week) {
//        case "1":
//            return"星期天";
//        case "2":
//            return"星期一";
//        case "3":
//            return"星期二";
//        case "4":
//            return"星期三";
//        case "5":
//            return"星期四";
//        case "6":
//            return"星期五";
//        case "7":
//            return"星期六";
//        default:
//            return null;
//        }
//    }

    /**
     * 描述
     * @param s
     * @param sBuffer
     * @author Norton Lai
     * @created 2019-2-27 下午5:00:15
     */
    private static void descMonth(String s, StringBuffer sBuffer) {
        desc(s, sBuffer, "月");
    }

    // 测试方法
    public static void main(String[] args) {
        String CRON_EXPRESSION = "0 0 0/2 * * ?";
       // String CRON_EXPRESSION = "* 5-8 1,5,7 * * ?";
//        String CRON_EXPRESSION = "0 0 5-8 * * ?";

        System.out.println(descCorn(CRON_EXPRESSION));
    }


  /*  public static void main(String[] args) {
        String CRON_EXPRESSION = "0 0/15 * ? * *";
        System.out.println(CRON_EXPRESSION);
        List<String> lTime = new ArrayList<String>();
        lTime = parser(CRON_EXPRESSION,5);
        for (int i = 0; i < lTime.size(); i++) {
            System.out.println(lTime.get(i));
        }
    }*/

}