package com.sunwin.swismp.util;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description:
 * @Author: fengyong
 * @CreateDate: 2022/4/29
 * @Remark:  节假日-工作日-非工作日
 * @Version: 1.0
 */
public class HolidayUtil {
    //http://www.apihubs.cn/#/holiday
    //https://api.apihubs.cn/holiday/get?field=date&month=202204&workday=2&order_by=1&cn=1&size=35
    //{"code":0,"msg":"ok","data":{"list":[{"date":20220403,"date_cn":"2022年04月03日"},{"date":20220404,"date_cn":"2022年04月04日"},{"date":20220405,"date_cn":"2022年04月05日"},{"date":20220409,"date_cn":"2022年04月09日"},{"date":20220410,"date_cn":"2022年04月10日"},{"date":20220416,"date_cn":"2022年04月16日"},{"date":20220417,"date_cn":"2022年04月17日"},{"date":20220423,"date_cn":"2022年04月23日"},{"date":20220430,"date_cn":"2022年04月30日"}],"page":1,"size":35,"total":9}}


    /**
     * 传入格式 为 yyyyMM 日期
     * @param date
     * @param type
     * @return
     */
    public static List<String> getDayList(String date,String type) {
        try {
            String filePath = "";
            if ("non_working".equals(type)){
                //非工作日
                filePath="https://api.apihubs.cn/holiday/get?field=date&month="+ date+"&workday=2&order_by=1&cn=1&size=31";
            }else if ("weekend".equals(type)){
                //周末
                filePath="https://api.apihubs.cn/holiday/get?field=date&month="+date+"&weekend=1&order_by=1&cn=1&size=31";
            }
            URL u = new URL(filePath);
            InputStream in = u.openStream();
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            try {
                byte buf[] = new byte[1024];
                int read = 0;
                while ((read = in.read(buf)) > 0) {
                    out.write(buf, 0, read);
                }
            } finally {
                if (in != null) {
                    in.close();
                }
            }
            byte b[] = out.toByteArray();
            String s = new String(b, "utf-8");
            JSONObject jsonObject = JSONObject.parseObject(s);
            JSONObject data = jsonObject.getJSONObject("data");
            JSONArray jsonArray = data.getJSONArray("list");
            List<Map> maps = jsonArray.toJavaList(Map.class);
            List<String> stringList = new ArrayList<>();
            for (int i=0;i<maps.size();i++){
                Map<String,Object> map = maps.get(i);
                stringList.add(TimeUtils.dateFormat(map.get("date").toString(),TimeUtils.YYYYMMDD,TimeUtils.YYYY_MM_DD));
            }
            return stringList;
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("------------------调用获取日期接口失败------------------");
            return null;
        }
    }

    //获取周末  月从1开始
    public static List<String> getMonthWekDay(int year,int mouth){
        List<String> dateList = new ArrayList<>();
        SimpleDateFormat simdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = new GregorianCalendar(year, mouth-1 , 1);
        Calendar endCalendar = new GregorianCalendar(year, mouth-1 , 1);
        endCalendar.add(Calendar.MONTH,1);
        while (true) {
            int weekday=calendar.get(Calendar.DAY_OF_WEEK);
            if(weekday == 1 || weekday== 7){
                dateList.add(simdf.format(calendar.getTime()));
            }
            calendar.add(Calendar.DATE,1);
            if (calendar.getTimeInMillis() >= endCalendar.getTimeInMillis()){
                break;
            }
        }
        return dateList;
    }


    /**
     * 计算周六日
     * @param args
     * @throws ParseException
     */
    public static void main(String[] args) throws ParseException {
       // List<String> monthWekDay = getMonthWekDay(2023, 5);
        String month = "202205";
        List<String> monthWekDay =  getMonthWekDay(Integer.parseInt(month.substring(0,4)), Integer.parseInt(month.substring(4)));
        for (String str:monthWekDay){
            System.out.println(str);
        }
    }
}
