package com.sunwin.swismp.util;

import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.PostMethod;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.*;
import java.util.Map;

/**
* @Description:
* @Author: zc
* @CreateDate: 2018/12/18 16:44
* @Remark:  查询天气的接口，捕获异常
* @Version: 1.0
*/
public class HttpUtil {
    public static String doGet(String httpUrl) {
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(httpUrl);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("Content-Type","application/json");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent","Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            //读取结果
            in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }



	public static String doPost(String httpUrl, String json) {
//        PrintWriter out = null;
		OutputStream os = null;
		BufferedReader in = null;
		String result = "";
		try {
			URL realUrl = new URL(httpUrl);
			// 打开和URL之间的连接
			URLConnection conn = realUrl.openConnection();
			// 设置通用的请求属性
			conn.setRequestProperty("accept", "*/*");
			conn.setRequestProperty("Content-Type","application/json");
			conn.setRequestProperty("connection", "Keep-Alive");
			conn.setRequestProperty("user-agent","Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
			// 发送POST请求必须设置如下两行
			conn.setDoOutput(true);
			conn.setDoInput(true);
			//1.获取URLConnection对象对应的输出流
//            out = new PrintWriter(conn.getOutputStream());
			//2.中文有乱码的需要将PrintWriter改为如下
			os = conn.getOutputStream();
			//out=new OutputStreamWriter(conn.getOutputStream(),"UTF-8")
			// 发送请求参数
			os.write(json.getBytes());

			// flush输出流的缓冲
			os.flush();
			// 定义BufferedReader输入流来读取URL的响应
			in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			String line;
			while ((line = in.readLine()) != null) {
				result += line;
			}
		} catch (Exception e) {
			System.out.println("发送 POST 请求出现异常！"+e);
			e.printStackTrace();
		}
		//使用finally块来关闭输出流、输入流
		finally{
			try{
				if(os!=null){
					os.close();
				}
				if(in!=null){
					in.close();
				}
			}
			catch(IOException ex){
				ex.printStackTrace();
			}
		}
		return result;
	}

    /**
     * POST（x-www-form-urlencoded）请求
     * @param httpUrl
     * @param params
     * @return
     */
    public static String doPost(String httpUrl, Map<String,Object> params) {
        try {
            PostMethod postMethod = new PostMethod(httpUrl) ;
            postMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8") ;
            postMethod.setRequestHeader("X-Requested-With","XMLHttpRequest");
            //参数设置，需要注意的就是里边不能传NULL，要传空字符串
            NameValuePair[] data= new NameValuePair[params.size()];
            int i=0;
            for (Map.Entry<String, Object> map :  params.entrySet()){
                data[i] = new NameValuePair(map.getKey(), (String) map.getValue());
                i++;
            }
            postMethod.setRequestBody(data);
            org.apache.commons.httpclient.HttpClient httpClient = new org.apache.commons.httpclient.HttpClient();
            int response = httpClient.executeMethod(postMethod); // 执行POST方法
            String result = postMethod.getResponseBodyAsString() ;

            return result;
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }
    }

	public static String getEncoding(String str){
		String encoding = "UTF-8";
		try {
			if (str.equals(new String(str.getBytes(),encoding))) {
				return encoding;
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}

		encoding = "GBK";
		try {
			if (str.equals(new String(str.getBytes(),encoding))) {
				return encoding;
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}

		encoding = "ISO-8859-1";
		try {
			if (str.equals(new String(str.getBytes(),encoding))) {
				return encoding;
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}

		encoding = "GB2312";
		try {
			if (str.equals(new String(str.getBytes(),encoding))) {
				return encoding;
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return null;
	}

    /**
     * 获取客户端ip
     * @param request
     * @return
     */
    public static String getIpAddr(HttpServletRequest request) {
        String ipAddress = null;
        try {
            ipAddress = request.getHeader("x-forwarded-for");
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("WL-Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getRemoteAddr();
                if (ipAddress.equals("127.0.0.1")) {
                    // 根据网卡取本机配置的IP
                    InetAddress inet = null;
                    try {
                        inet = InetAddress.getLocalHost();
                    } catch (UnknownHostException e) {
                        e.printStackTrace();
                    }
                    ipAddress = inet.getHostAddress();
                }
            }
            // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
            if (ipAddress != null && ipAddress.length() > 15) { // "***.***.***.***".length()
                // = 15
                if (ipAddress.indexOf(",") > 0) {
                    ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
                }
            }
        } catch (Exception e) {
            ipAddress="";
        }
        // ipAddress = this.getRequest().getRemoteAddr();

        return ipAddress;
    }

}
