package com.sunwin.swismp.util;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <br>
 * Copyright (c) 2019/11/13 All Rights Reserved By SUNWIN<br>
 * <br>
 * Package: com.sunwin.swbd.utils <br>
 * FileName: NumberUtil <br>
 * <br>
 *
 * @author duanmin
 * @created 2019/11/13 15:48
 * @last Modified
 * @history
 */
public class NumberUtils {

    /**
     * 返回带单位的数值
     *
     * @param originNum
     * @return
     */
    public static DecimalFormat df = new DecimalFormat("0.0");//设置保留位数

    public static String getNumWithUnit(String originNum) {
        if (Long.parseLong(originNum) > 100000000) {
            BigDecimal bigDecimal = new BigDecimal(Long.parseLong(originNum) / 100000000.0d);
            return (bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).toString() + "亿");
        } else if (Long.parseLong(originNum) > 10000) {
            BigDecimal bigDecimal = new BigDecimal(Long.parseLong(originNum) / 10000.0d);
            return (bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).toString() + "万");
        }
        return originNum;
    }

    /**
     * 返回带单位的条目数值
     *
     * @param originNum
     * @return
     */
    public static String getNumWithUnitByInt(Long originNum) {
        if (originNum > 100000000) {
            BigDecimal bigDecimal = new BigDecimal(originNum / 100000000.0d);
            return (bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).toString() + "亿");
        } else if (originNum > 10000) {
            BigDecimal bigDecimal = new BigDecimal(originNum / 10000.0d);
            return (bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).toString() + "万");
        }
        return originNum + "";
    }

    /**
     * 将double形式的字符串转成long型
     *
     * @param a
     * @return
     */
    public static Long doubleToLong(Object a) {
        return Math.round(Double.valueOf(String.valueOf(a)));
    }

    /**
     * 返回带单位的Map条目数值
     *
     * @param originNum
     * @return
     */
    public static Map getMapNumWithUnitByInt(Long originNum) {
        Map map = new HashMap();
        if (originNum > 100000000) {
            BigDecimal bigDecimal = new BigDecimal(originNum / 100000000.0d);
            map.put("unit", "亿");
            map.put("data", bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP));
        } else if (originNum > 10000) {
            BigDecimal bigDecimal = new BigDecimal(originNum / 10000.0d);
            map.put("unit", "万");
            map.put("data", bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP));
        } else {
            map.put("unit", "");
            map.put("data", originNum);
        }
        return map;
    }


    /**
     * 返回带单位的大小数值
     *
     * @param a
     * @return
     */
    public static String getSizeWithUnit(Long a) {
        if (a >= 1048576L && a < 1073741824L) {
            return df.format(a / 1048576.0) + "M";
        } else if (a >= 1073741824L) {
            return df.format((a / 1048576.0) / 1024.0) + "G";
        } else {
            return df.format(a / 1024.0) + "K";
        }
    }

    /**
     * 返回带单位的Map大小数值
     *
     * @param a
     * @return
     */
    public static Map getMapSizeWithUnit(Long a) {
        Map map = new HashMap();
        if (a >= 1048576L && a < 1073741824L) {
            map.put("unit", "MB");
            map.put("data", df.format(a / 1048576.0));
        } else if (a >= 1073741824L && a < 1024 * 1024 * 1024 * 1024L) {
            map.put("unit", "GB");
            map.put("data", df.format((a / 1048576.0) / 1024.0));
        } else if (a >= 1024 * 1024 * 1024 * 1024L) {
            map.put("unit", "TB");
            map.put("data", df.format((a / 1048576.0) / 1024.0 / 1024.0));
        } else {
            map.put("unit", "KB");
            map.put("data", df.format((a) / 1024.0));
        }
        return map;
    }

    /**
     * 将嵌套数组按照最大值统一单位
     */
    public static Map getSizeMultiListWithUnit(List<List<Double>> sizes) {
        String unit = "kb";
        Map mapTotal = new HashMap();
        List dataTotal = new ArrayList();
        for (List<Double> mem : sizes) {
            for (Double size : mem) {
                if (size != null) {
                    if (size > 1073741824) {
                        unit = !unit.equals("tb") ? "gb" : unit;
                    } else if (size >= 1048576 && size < 1073741824) {
                        unit = unit.equals("kb") ? "mb" : unit;
                    } else if (size > 1024 * 1024 * 1024 * 1024L) {
                        unit = "tb";
                    }
                }
            }
        }
        for (List<Double> mem : sizes) {
            Map map = computeSizeByUnit(mem, unit);
            mapTotal.put("unit", map.get("unit"));
            List memData = (List) map.get("list");
            dataTotal.add(memData);
        }
        mapTotal.put("list", dataTotal);
        return mapTotal;
    }

    /**
     * 将数组按照最大条数的统一单位
     */
    public static Map getNumListWithUnit(List<Long> nums) {
        String unit = "";
        Map map = new HashMap<>();
        for (Long num : nums) {
            if (num != null) {
                if (num > 100000000) {
                    unit = "亿";
                } else if (num > 10000) {
                    unit = unit.equals("") ? "万" : unit;
                }
            }
        }
        switch (unit) {
            case "万":
                map.put("unit", "万");
                map.put("list", computeNumByUnit(nums, unit));
                break;
            case "亿":
                map.put("unit", "亿");
                map.put("list", computeNumByUnit(nums, unit));
                break;
            default:
                map.put("unit", "");
                map.put("list", nums);
        }
        return map;
    }

    /**
     * 将数组按照最大条数的统一单位
     */
    public static Map getNumListWithUnitDouble(List<Double> nums) {
        String unit = "";
        Map map = new HashMap<>();
        for (Double num : nums) {
            if (num > 100000000) {
                unit = "亿";
            } else if (num > 10000) {
                unit = unit.equals("") ? "万" : unit;
            }
        }
        switch (unit) {
            case "万":
                map.put("unit", "万");
                map.put("list", computeNumByUnitDouble(nums, unit));
                break;
            case "亿":
                map.put("unit", "亿");
                map.put("list", computeNumByUnitDouble(nums, unit));
                break;
            default:
                map.put("unit", "");
                map.put("list", nums);
        }
        return map;
    }

    /**
     * 将数组按照最大大小的统一单位
     */
    public static Map getSizeListWithUnit(List<Double> sizes) {
        String unit = "kb";
        for (Double size : sizes) {
            if (size != null) {
                if (size > 1073741824) {
                    unit = !unit.equals("tb") ? "gb" : unit;
                } else if (size >= 1048576 && size < 1073741824) {
                    unit = unit.equals("kb") ? "mb" : unit;
                } else if (size > 1024 * 1024 * 1024 * 1024L) {
                    unit = "tb";
                }
            }
        }
        Map map = computeSizeByUnit(sizes, unit);
        return map;
    }

    /**
     * @param sizes
     * @param unit
     * @Description : 计算 G M K 级别的数据
     * @Return :
     * @Author : yangdong
     * @Date : 2020/4/16 13:22
     */
    private static Map computeSizeByUnit(List<Double> sizes, String unit) {
        List<Double> numNew = new ArrayList<>();
        Map map = new HashMap();
        for (Double size : sizes) {
            if (size != null) {
                BigDecimal bigDecimal = null;
                if (unit.equals("gb")) {
                    bigDecimal = new BigDecimal((size / 1048576.0) / 1024.0);
                } else if (unit.equals("mb")) {
                    bigDecimal = new BigDecimal(size / 1048576.0);
                } else if (unit.equals("kb")) {
                    bigDecimal = new BigDecimal(size / 1024.0);
                } else if (unit.equals("tb")) {
                    bigDecimal = new BigDecimal((size / 1048576.0) / 1024.0 / 1024.0);
                }
                numNew.add(bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            } else {
                numNew.add(null);
            }
        }
        map.put("unit", unit);
        map.put("list", numNew);
        return map;
    }

    /**
     * @param unit
     * @Description : 根据单位大小 返回要除以的转换值
     * @Return :
     * @Author : yangdong
     * @Date : 2020/6/3 10:28
     */
    public static Integer convertValueBySizeUnit(String unit) {
        Integer convertValue = 1;
        switch (unit) {
            case "kb":
                convertValue = 1024;
                break;
            case "mb":
                convertValue = 1024 * 1024;
                break;
            case "gb":
                convertValue = 1024 * 1024 * 1024;
                break;
            case "tb":
                convertValue = 1024 * 1024 * 1024 * 1024;
        }
        return convertValue;
    }

    /**
     * @param nums
     * @param unit
     * @Description : 计算 亿 万 级别的数据
     * @Return :
     * @Author : yangdong
     * @Date : 2020/4/16 13:22
     */
    private static List computeNumByUnit(List<Long> nums, String unit) {
        List<Double> numNew = new ArrayList<>();
        for (Long num : nums) {
            if (num != null) {
                BigDecimal bigDecimal = null;
                if (unit.equals("亿")) {
                    bigDecimal = new BigDecimal(num / 100000000.0d);
                } else if (unit.equals("万")) {
                    bigDecimal = new BigDecimal(num / 10000.0d);
                }
                numNew.add(bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            } else {
                numNew.add(null);
            }

        }
        return numNew;

    }

    /**
     * @param nums
     * @param unit
     * @Description : 计算 亿 万 级别的数据
     * @Return :
     * @Author : yangdong
     * @Date : 2020/4/16 13:22
     */
    private static List computeNumByUnitDouble(List<Double> nums, String unit) {
        List<Double> numNew = new ArrayList<>();
        for (Double num : nums) {
            if (num != null) {
                BigDecimal bigDecimal = null;
                if (unit.equals("亿")) {
                    bigDecimal = new BigDecimal(num / 100000000.0d);
                } else if (unit.equals("万")) {
                    bigDecimal = new BigDecimal(num / 10000.0d);
                }
                numNew.add(bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            } else {
                numNew.add(null);
            }
        }
        return numNew;

    }

    /**
     * @param unit
     * @Description : 根据单位大小 返回要除以的转换值
     * @Return :
     * @Author : yangdong
     * @Date : 2020/6/3 10:28
     */
    public static Integer convertValueByNumUnit(String unit) {
        Integer convertValue = 1;
        switch (unit) {
            case "万":
            case "万条":
            case "万个":
                convertValue = 10000;
                break;
            case "亿":
            case "亿条":
            case "亿个":
                convertValue = 10000 * 10000;
                break;
        }
        return convertValue;
    }

    /**
     * @param d
     * @param i
     * @Description : 对double对象四舍五入 保留指定位数的小数
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/31 22:35
     */
    public static Double setScaleDouble(Double d, int newScale) {
        String pattern = "0.";
        for (int i = 0; i < newScale; i++) {
            pattern += "0";
        }
        DecimalFormat format = new DecimalFormat(pattern);
        BigDecimal b = new BigDecimal(d);
        Double f = b.setScale(newScale, BigDecimal.ROUND_HALF_UP).doubleValue();
        return Double.valueOf(format.format(b));
    }

    /**
     * @param d
     * @param i
     * @Description : 对float对象四舍五入 保留指定位数的小数
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/31 22:35
     */
    public static Float setScaleFloat(Float d, int newScale) {
        String pattern = "0.";
        for (int i = 0; i < newScale; i++) {
            pattern += "0";
        }
        DecimalFormat format = new DecimalFormat(pattern);
        BigDecimal b = new BigDecimal(d);
        Float f = b.setScale(newScale, BigDecimal.ROUND_HALF_UP).floatValue();
        return Float.valueOf(format.format(b));
    }

    /**
     * 保留两位小数，小数不足两位补0
     *
     * @param d
     * @param i
     * @return
     */
    public static String getTwoDigitDouble(Double d, int i) {
        if (d == null){
            d = 0.0;
        }
        String pattern = "#0.";
        int n = i;
        while (i > 0) {
            pattern += "0";
            i--;
        }
        DecimalFormat format = new DecimalFormat(pattern);
        BigDecimal b = new BigDecimal(d);
        Double f = b.setScale(n, BigDecimal.ROUND_HALF_UP).doubleValue();
        return format.format(f);
    }

    /**
     * @param d
     * @param i
     * @return
     */
    public static Double getDigitDouble(Double d, int i) {
        BigDecimal b = new BigDecimal(d);
        Double f = b.setScale(i, BigDecimal.ROUND_HALF_UP).doubleValue();
        return f;
    }

    /**
     * @param arr       数值列表
     * @param idx       所求数占比的索引
     * @param precision 保留小数点位数
     * @Description : 最大余数法 求占比
     * @Return :
     * @Author : yangdong
     * @Date : 2020/4/3 10:25
     */
    public static String getPercentValue(List<Double> arr, int idx, int precision) {
        if ((arr.size() - 1) < idx) {
            return "0.00%";
        }
        //求和
        Double sum = 0.0;
        for (int i = 0; i < arr.size(); i++) {
            sum += arr.get(i);
        }
        //10的2次幂是100，用于计算精度。
        double digits = Math.pow(10, precision);
        //扩大比例100
        double[] votesPerQuota = new double[arr.size()];
        for (int i = 0; i < arr.size(); i++) {
            double val = arr.get(i) / sum * digits * 100;
            votesPerQuota[i] = val;
        }
        //总数,扩大比例意味的总数要扩大
        double targetSeats = digits * 100;
        //再向下取值，组成数组
        double[] seats = new double[arr.size()];
        for (int i = 0; i < votesPerQuota.length; i++) {
            seats[i] = Math.floor(votesPerQuota[i]);
        }
        //再新计算合计，用于判断与总数量是否相同,相同则占比会100%
        double currentSum = 0;
        for (int i = 0; i < seats.length; i++) {
            currentSum += seats[i];
        }
        //余数部分的数组:原先数组减去向下取值的数组,得到余数部分的数组
        double[] remainder = new double[arr.size()];
        for (int i = 0; i < seats.length; i++) {
            remainder[i] = votesPerQuota[i] - seats[i];
        }
        while (currentSum < targetSeats) {
            double max = 0;
            int maxId = 0;
            int len = 0;
            for (int i = 0; i < remainder.length; ++i) {
                if (remainder[i] > max) {
                    max = remainder[i];
                    maxId = i;
                }
            }
            //对最大项余额加1
            ++seats[maxId];
            //已经增加最大余数加1,则下次判断就可以不需要再判断这个余额数。
            remainder[maxId] = 0;
            //总的也要加1,为了判断是否总数是否相同,跳出循环。
            ++currentSum;
        }
        // 这时候的seats就会总数占比会100%
        return getTwoDigitDouble(seats[idx] / digits, precision) + "%";
    }

    /**
     * @param arr       数值列表
     * @param idx       所求数占比的索引
     * @param precision 保留小数点位数
     * @Description : 最大余数法 求占比
     */
    public static Double getPercentValueDoubue(List<Double> arr, int idx, int precision) {
        if ((arr.size() - 1) < idx) {
            return 0.00d;
        }
        //求和
        Double sum = 0.0;
        for (int i = 0; i < arr.size(); i++) {
            sum += arr.get(i);
        }
        //10的2次幂是100，用于计算精度。
        double digits = Math.pow(10, precision);
        //扩大比例100
        double[] votesPerQuota = new double[arr.size()];
        for (int i = 0; i < arr.size(); i++) {
            double val = arr.get(i) / sum * digits * 100;
            votesPerQuota[i] = val;
        }
        //总数,扩大比例意味的总数要扩大
        double targetSeats = digits * 100;
        //再向下取值，组成数组
        double[] seats = new double[arr.size()];
        for (int i = 0; i < votesPerQuota.length; i++) {
            seats[i] = Math.floor(votesPerQuota[i]);
        }
        //再新计算合计，用于判断与总数量是否相同,相同则占比会100%
        double currentSum = 0;
        for (int i = 0; i < seats.length; i++) {
            currentSum += seats[i];
        }
        //余数部分的数组:原先数组减去向下取值的数组,得到余数部分的数组
        double[] remainder = new double[arr.size()];
        for (int i = 0; i < seats.length; i++) {
            remainder[i] = votesPerQuota[i] - seats[i];
        }
        while (currentSum < targetSeats) {
            double max = 0;
            int maxId = 0;
            int len = 0;
            for (int i = 0; i < remainder.length; ++i) {
                if (remainder[i] > max) {
                    max = remainder[i];
                    maxId = i;
                }
            }
            //对最大项余额加1
            ++seats[maxId];
            //已经增加最大余数加1,则下次判断就可以不需要再判断这个余额数。
            remainder[maxId] = 0;
            //总的也要加1,为了判断是否总数是否相同,跳出循环。
            ++currentSum;
        }
        // 这时候的seats就会总数占比会100%
        return getDigitDouble(seats[idx] / digits, precision);
    }
}
