package com.sunwin.swismp.util;

import com.sunwin.swismp.constant.TimeIntervalEnum;
import com.sunwin.swismp.constant.TimeIntervalSeperator;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * <br>
 * Copyright (c) 2019/11/13 All Rights Reserved By SUNWIN<br>
 * <br>
 * Package: com.sunwin.swbd.utils <br>
 * FileName: TimeUtil <br>
 * <br>
 * 时间工具类
 *
 * @author duanmin
 * @created 2019/11/13 15:48
 * @last Modified
 * @history
 */
public class TimeUtils {

    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static final String YYYYMMDD = "yyyyMMdd";
    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
    public static final String START_TIMESTAMP = " 00:00:00";
    public static final String END_TIMESTAMP = " 23:59:59";
    public static final String HHMISS = "HHmmss";
    public static final String HH_MI = "HH:mm";

    public static final String[] TIME_SUFFIX = {"点", "日", "月"};


    /**
     * 设置修改时间时分秒
     * @param date
     * @param hour
     * @param min
     * @param sec
     * @return
     */
    public static Date getDateSetHourMinSec(Date date,Integer hour,Integer min,Integer sec){
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date);
        cal1.set(Calendar.HOUR_OF_DAY, hour);
        cal1.set(Calendar.MINUTE, min);
        cal1.set(Calendar.SECOND, sec);
        return cal1.getTime();
    }

    /**
     * 获取该时间的前几分钟时间
     *
     * @param date
     * @param minute
     * @param dateFormat
     * @return
     */
    public static String getbeforeTime(Date date, Integer minute, String dateFormat) {
        Calendar beforeTime = Calendar.getInstance();
        beforeTime.setTime(date);
        beforeTime.add(Calendar.MINUTE, -1 * minute);// 5分钟之前的时间
        Date beforeD = beforeTime.getTime();
        String before = new SimpleDateFormat(dateFormat).format(beforeD);
        return before;
    }

    /**
     * 工作日判断
     *
     * @param calendar
     * @return
     */
    public static boolean checkHoliday(Calendar calendar) {
        if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY ||
                calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
            return true;
        }
        return false;
    }

    /**
     * 连续一段时间判断有无工作日
     * 返回0，说明都是非工作日，
     * 返回1，说明都是工作日，
     * 返回2，说明有工作日有非工作日
     *
     * @param beginDate
     * @param endDate
     * @param dateFormat
     * @return
     */
    public static int checkWorkingDays(String beginDate, String endDate, String dateFormat) {
        Calendar beginCalendar = Calendar.getInstance();
        beginCalendar.setTime(getDateFromStr(beginDate, dateFormat));
        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(getDateFromStr(endDate, dateFormat));
        int flag = 0;
        int index = 0;
        while (beginCalendar.compareTo(endCalendar) <= 0) {
            if (checkHoliday(beginCalendar)) {
                flag += 0;
            } else {
                flag += 1;
            }
            beginCalendar.add(Calendar.DATE, 1);
            if (index++ > 5) {
                return 2;
            }
        }
        if (flag == 0) {
            return 0;
        } else if (flag == index) {
            return 1;
        } else {
            return 2;
        }
    }

    /**
     * 将字符串类型的日期转换成date类型
     *
     * @param dateStr
     * @param dateFormat
     * @return
     * @throws ParseException
     */
    public static Date getDateFromStr(String dateStr, String dateFormat) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dateFormat);
        try {
            return simpleDateFormat.parse(dateStr);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 将字符串日期格式从一种格式转换成另一种格式，并将结果返回
     *
     * @param dateStr
     * @param originDateFormat
     * @param targetDateFormat
     * @return
     */
    public static String getStrFromStr(String dateStr, String originDateFormat, String targetDateFormat) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat(originDateFormat);
        SimpleDateFormat dateFormat2 = new SimpleDateFormat(targetDateFormat);
        try {
            return dateFormat2.format(dateFormat1.parse(dateStr));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 距离current add天前（后）
     *
     * @param current
     * @param add
     * @return
     */
    public static Date preDate(Date current, Integer add, Integer type) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(current);
        if (type == 0) { //天
            calendar.add(Calendar.DAY_OF_MONTH, add);
        } else if (type == 1) {
            calendar.add(Calendar.MONTH, add);
        } else if (type == 2) {
            calendar.add(Calendar.YEAR, add);
        } else {
            return null;
        }
        return calendar.getTime();
    }

    /**
     * 获取第几周的第一天日期（星期一）
     *
     * @param year
     * @param weekNo
     * @return
     */
    public static Date weekFirstDay(Integer year, Integer weekNo) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.WEEK_OF_YEAR, weekNo);
        return calendar.getTime();
    }

    /**
     * 获取第几周的最后一天日期（星期天)
     *
     * @param year
     * @param weekNo
     * @return
     */
    public static Date weekEndDay(Integer year, Integer weekNo) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.WEEK_OF_YEAR, weekNo);
        calendar.add(Calendar.DAY_OF_WEEK, 6);
        return calendar.getTime();
    }

    /**
     * 获取日期间隔
     *
     * @param beginDate
     * @param endDate
     * @return
     */
    public static int dateInterval(Date beginDate, Date endDate) {
        Calendar beginCalendar = Calendar.getInstance();
        beginCalendar.setTime(beginDate);
        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(endDate);
        long beginTime = beginCalendar.getTime().getTime();
        long endTime = endCalendar.getTime().getTime();
        int betweenDays = (int) ((endTime - beginTime) / (1000 * 60 * 60 * 24));//先算出两时间的毫秒数之差大于一天的天数

        endCalendar.add(Calendar.DAY_OF_MONTH, -betweenDays);//使endCalendar减去这些天数，将问题转换为两时间的毫秒数之差不足一天的情况
        endCalendar.add(Calendar.DAY_OF_MONTH, -1);//再使endCalendar减去1天
        if ((beginCalendar.get(Calendar.DAY_OF_MONTH) == endCalendar.get(Calendar.DAY_OF_MONTH))) {//比较两日期的DAY_OF_MONTH是否相等
            return betweenDays + 1;
        }//相等说明确实跨天了
        else {
            return betweenDays + 0;
        }
    }


    /**
     * 获取开始时间
     *
     * @param endDate
     * @param dateType
     * @return
     */
    public static Date getBeginDate(Date endDate, Integer dateType) {
        if (dateType == 0) { //最近一周
            return preDate(endDate, -6, 0);
        } else if (dateType == 1) { //最近一月
            return preDate(endDate, -1, 1);
        } else if (dateType == 2) {
            return preDate(endDate, -1, 2);
        }
        return null;
    }


    /**
     * 星期几，星期一：1
     *
     * @param date
     * @return
     */
    public static Integer dayOfWeek(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int dw = calendar.get(Calendar.DAY_OF_WEEK);
        int[] dws = {7, 1, 2, 3, 4, 5, 6};
        return dws[dw - 1];
    }

//    /**
//     * 获取该月份所有的日期
//     *
//     * @param year
//     * @param month
//     * @return
//     */
//    public static List<String> getMonthEveryDays(int year, int month) {
//        int days = getDaysByYearMonth(year, month);
//        List<String> list = new ArrayList<>();
//        for (int i = 1; i <= days; i++) {
//            list.add(String.valueOf(year).concat("-")
//                    .concat(org.apache.commons.lang.StringUtils.leftPad(String.valueOf(month), 2, "0").concat("-")
//                            .concat(org.apache.commons.lang.StringUtils.leftPad(String.valueOf(i), 2, "0"))));
//        }
//        return list;
//    }

    /**
     * 获取年 月的天数
     *
     * @param year
     * @param month
     * @return
     */
    public static int getDaysByYearMonth(int year, int month) {
        Calendar a = Calendar.getInstance();
        a.set(Calendar.YEAR, year);
        a.set(Calendar.MONTH, month - 1);
        a.set(Calendar.DATE, 1);
        a.roll(Calendar.DATE, -1);
        int maxDate = a.get(Calendar.DATE);
        return maxDate;
    }


    /**
     * 转换字符串日期格式
     *
     * @param date
     * @param sourceFormat
     * @param targetFormat
     * @return
     * @throws ParseException
     */
    public static String dateFormat(String date, String sourceFormat, String targetFormat) {
        if (date == null) {
            return null;
        }
        DateFormat df1 = new SimpleDateFormat(sourceFormat);
        DateFormat df2 = new SimpleDateFormat(targetFormat);
        try {
            return df2.format(df1.parse(date));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 返回时间间隔内所有日期，yyyy-MM-dd,闭合区间
     *
     * @param beginDate
     * @param endDate
     * @return
     */
    public static List<String> getDates(Date beginDate, Date endDate, String format) {
        DateFormat df = new SimpleDateFormat(format);
        Integer days = TimeUtils.dateInterval(beginDate, endDate);
        List<String> dates = new ArrayList<>();
        for (int i = 0; i <= days; i++) {
            Date temp = TimeUtils.preDate(beginDate, i, 0);
            dates.add(df.format(temp));
        }
        return dates;
    }


    /**
     * 获取指定月份最后一天
     *
     * @param yearMonth yyyy-MM
     * @return
     */
    public static String getLastDayOfMonth(String yearMonth) {
        int year = Integer.parseInt(yearMonth.split("-")[0]);  //年
        int month = Integer.parseInt(yearMonth.split("-")[1]); //月
        Calendar cal = Calendar.getInstance();
        // 设置年份
        cal.set(Calendar.YEAR, year);
        // 设置月份
        // cal.set(Calendar.MONTH, month - 1);
        cal.set(Calendar.MONTH, month); //设置当前月的上一个月
        // 获取某月最大天数
        //int lastDay = cal.getActualMaximum(Calendar.DATE);
        int lastDay = cal.getMinimum(Calendar.DATE); //获取月份中的最小值，即第一天
        // 设置日历中月份的最大天数
        //cal.set(Calendar.DAY_OF_MONTH, lastDay);
        cal.set(Calendar.DAY_OF_MONTH, lastDay - 1); //上月的第一天减去1就是当月的最后一天
        // 格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(cal.getTime());
    }

    /**
     * 返回当天所有的时刻，HH:mm, 00:00-23:00
     *
     * @return
     */
    public static List<String> getHours() {
        List<String> hours = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            hours.add(StringUtils.leftPad(String.valueOf(i), 2, "0") + ":00");
        }
        return hours;
    }

    /**
     * @Description : 返回过去24小时的时间区间
     * @Return : Map now 表示的是现在时刻  before 过去24h的时刻  形式都为 yyyy-MM-dd HH:mm:ss
     * @Author : yangdong
     * @Date : 2020/4/15 14:56
     */
    public static Map<String, String> get24Hours() {
        Date now = new Date();
        Calendar nowCalendar = Calendar.getInstance();
        nowCalendar.setTime(now);
        /**
         * 如果此时时刻是59分 因为秒数不确定 可能求得的范围没有发送数据，所以改为58分
         */
        if (nowCalendar.get(Calendar.MINUTE) == 59) {
            nowCalendar.set(Calendar.MINUTE, 58);
            now = nowCalendar.getTime();
        }
        Calendar calendar = Calendar.getInstance();
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        calendar.setTime(now);
        calendar.add(Calendar.DAY_OF_YEAR, -1);
        Date before24Hours = calendar.getTime();
        Map map = new HashMap();
        map.put("before", format.format(before24Hours));
        map.put("now", format.format(now));
        return map;
    }

    /**
     * @param intervalEnum 对应时间间隔枚举类
     * @param date         当前日期 形式为yyy-MM-dd HH:mm:ss
     * @param markPosition 给图例标记的字符串 例如 开始  截止等 不填则无
     * @Description : 获取返回日期在图例中的表现形式
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/2 10:53
     */
    public static String getTimeCategory(TimeIntervalEnum intervalEnum, String date, String... markPosition) {
        String dateCategory = null;
        switch (intervalEnum) {
            case DAY:
                dateCategory = date.split(" ")[0];
                break;
            case HOUR:
                String[] hourCategorys = date.split(" ")[1].split(":");
                dateCategory = hourCategorys[0] + ":" + hourCategorys[1];
                break;
            /**
             * 分析该日期是否为月/年第一日 如果是 则不用处理
             * 如果不是 则 补充开始 截止日期
             */
            case MONTH:
                String[] monthCategorys = date.split("-");
                if (markPosition == null || markPosition.length == 0) {
                    dateCategory = monthCategorys[0] + "-" + monthCategorys[1];
                } else {
                    dateCategory = monthCategorys[0] + "-" + monthCategorys[1] + "(" + monthCategorys[1] + "-" + monthCategorys[2].split(" ")[0] + markPosition[0] + ")";
                }
                break;
            case YEAR:
                String[] yearCategorys = date.split("-");
                if (markPosition == null || markPosition.length == 0) {
                    dateCategory = yearCategorys[0];
                } else {
                    dateCategory = yearCategorys[0] + "(" + yearCategorys[1] + "-" + yearCategorys[2].split(" ")[0] + markPosition[0] + ")";
                }
                break;
        }
        return dateCategory;
    }




    /**
     * @param intervalEnum
     * @param begin
     * @param end
     * @Description : 获取当前间隔点的结束时间
     * @Return : 当前间隔点结束时间
     * @Author : yangdong
     * @Date : 2020/3/3 11:24
     */
    public static Calendar getSeperatorEnd(TimeIntervalEnum intervalEnum, Calendar begin, Calendar end) {
        /**
         * 如果开始日期到下一个间隔点 就大于结束日期
         * 说明开始日期与结束日期处于同一个间隔范围内，此时间隔点对应的结束
         * 时间就应为结束日期，若不是 则设置为间隔范围内的结束时间
         */
        begin.add(intervalEnum.getSeperatorInterval(), 1);
        Boolean endIsLast = begin.getTimeInMillis() > end.getTimeInMillis();
        begin.add(intervalEnum.getSeperatorInterval(), -1);
        if (endIsLast) {
            begin.add(intervalEnum.getSeperatorInterval(), -1);
            return end;
        } else {
            Calendar seperatorEnd = Calendar.getInstance();
            seperatorEnd.clear();
            seperatorEnd.setTime(begin.getTime());
            Integer count = begin.getActualMaximum(intervalEnum.getInnerInterval());
            seperatorEnd.set(intervalEnum.getInnerInterval(), count);
            if (intervalEnum != TimeIntervalEnum.HOUR) {
                seperatorEnd.set(Calendar.HOUR, 23);
                seperatorEnd.set(Calendar.MINUTE, 59);
            }
            seperatorEnd.set(Calendar.SECOND, 59);
            return seperatorEnd;
        }
    }

    /**
     * @param intervalEnum
     * @param begin
     * @Description : 获取当前日期的间隔范围内的开始时间
     * @Return :
     * @Author : yangdong
     * @Date : 2020/3/3 13:01
     */
    public static Calendar nextSeperatorBegin(TimeIntervalEnum intervalEnum, Calendar begin) {
        begin.add(intervalEnum.getSeperatorInterval(), 1);
        Calendar seperatorBegin = Calendar.getInstance();
        seperatorBegin.clear();
        Integer count = begin.getActualMinimum(intervalEnum.getInnerInterval());
        seperatorBegin.setTime(begin.getTime());
        seperatorBegin.set(intervalEnum.getInnerInterval(), count);
        if (intervalEnum != TimeIntervalEnum.HOUR) {
            seperatorBegin.set(Calendar.HOUR, 0);
            seperatorBegin.set(Calendar.MINUTE, 0);
        }
        seperatorBegin.set(Calendar.SECOND, 0);
        return seperatorBegin;
    }


    /**
     * @param dateFormat
     * @Description : 获取时间类型
     * @Return :
     * @Author : yangdong
     * @Date : 2020/4/24 18:47
     */
    public static String getTimeType(String dateFormat) {
        String timeType = "day";
        if (dateFormat.equals("HH:mm")) {
            timeType = "hour";
        } else if (dateFormat.equals("yyyy-MM-dd")) {
            timeType = "day";
        } else if (dateFormat.equals("yyyy-MM")) {
            timeType = "month";
        } else if (dateFormat.equals("yyyy")) {
            timeType = "year";
        }
        return timeType;
    }

    /**
     * 秒转换格式  22h 53m 52s
     * @return
     */
    public static String getStrSeconds(Long sNum) {
        Long n = sNum;
        Integer day = Math.toIntExact(n / 60 / 60 / 24);
        n = n-(day*60*60*24);
        Integer hour = Math.toIntExact( n/60 / 60);
        n= n-(hour*60*60);
        Integer min = Math.toIntExact( n/60);
        Integer s = Math.toIntExact(n - (min * 60));
        String str ="";
        if (day>0) str =str+day +"d ";
        if (hour>0)str = str+hour+"h ";
        if (min>0) str = str+min+"m ";
        str = str+s+"s";
        return str;
    }

    /**
     * 获取不同周期时间段
     * @param type 1.日 2.月 3.年
     * @param now 是否截止当前时间点
     * @return 时间段
     */
    public static List<Integer> getTimeSegment(int type, boolean now) {
        List<Integer> list = new ArrayList<>();
        int min = 0;
        int max = 0;
        if (type == 1){ // 日
            max = now ? LocalTime.now().getHour() : 23;
        }else if(type == 2){ // 月
            min = 1;
            LocalDate nowDate = LocalDate.now();
            max = now ? nowDate.getDayOfMonth() : nowDate.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
        }else if(type == 3){ // 年
            min = 1;
            max = now ? YearMonth.now().getMonthValue() : 12;
        }
        return IntStream.rangeClosed(min, max).boxed().collect(Collectors.toList());
    }

}
