package com.sunwin.swismp.bean;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Copyright (c) 2021/12/1 All Rights Reserved By SUNWIN
 * <p>
 * Package: com.sunwin.swurtmp.bean.baseinfo
 * FileName: TreeNode
 *
 * @author duanmin
 * @description
 * @created 2021/12/1-16:50
 * @last Modified
 * @history
 */
public class TreeNode implements Serializable {
    private static final long serialVersionUID = 1L;
    public static final int DEFAULT_ROOT = 0;

    private Integer id;//树结构节点id（父子节点都不可重复、唯一）
    private Integer nodeId;//节点数据id值，根据不同业务而定，父子节点对应含义可能不同（如 父节点 线路id， 子节点 站点id）
    private String title;
    private Integer level;//层级 （0开始）
    private boolean expand;//是否展开
    private boolean disabled;//是否可点击
    private boolean selected;//是否选中
    private boolean isParent;//是否是父节点

    /** 是否支持右键菜单 */
    private boolean contextmenu;

    /** 拓展属性 */
    @JsonInclude(JsonInclude.Include.NON_NULL)
    private Map<String, Object> extra;

    /** 子节点集合 */
    private List<TreeNode> children = new ArrayList<>();


    /** 父ID */
    @JsonIgnore
    private Integer parentId;

    public TreeNode() {
    }

    public TreeNode(Integer id, Integer nodeId, String title, Integer level) {
        this.id = id;
        this.nodeId = nodeId;
        this.title = title;
        this.level = level;
    }

    public TreeNode(Integer id, Integer parentId, Integer nodeId, String title, Integer level, boolean contextmenu) {
        this.id = id;
        this.parentId = parentId;
        this.nodeId = nodeId;
        this.title = title;
        this.level = level;
        this.contextmenu = contextmenu;
    }

    public TreeNode(Integer id, Integer nodeId, String title, boolean expand, boolean disabled, boolean selected,
                    boolean isParent, Integer level) {
        this.id = id;
        this.nodeId = nodeId;
        this.title = title;
        this.expand = expand;
        this.disabled = disabled;
        this.selected = selected;
        this.isParent = isParent;
        this.level = level;
    }

    public void addChildren(TreeNode node) {
        if (this.children == null){
            this.children = new ArrayList<>();
        }
        this.children.add(node);
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public boolean isExpand() {
        return expand;
    }

    public void setExpand(boolean expand) {
        this.expand = expand;
    }

    public boolean isDisabled() {
        return disabled;
    }

    public void setDisabled(boolean disabled) {
        this.disabled = disabled;
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }

    public boolean isParent() {
        return isParent;
    }

    public void setParent(boolean parent) {
        isParent = parent;
    }

    public List<TreeNode> getChildren() {
        return children;
    }

    public void setChildren(List<TreeNode> children) {
        this.children = children;
    }

    public Integer getNodeId() {
        return nodeId;
    }

    public void setNodeId(Integer nodeId) {
        this.nodeId = nodeId;
    }

    public Integer getLevel() {
        return level;
    }

    public void setLevel(Integer level) {
        this.level = level;
    }

    public boolean isContextmenu() {
        return contextmenu;
    }

    public void setContextmenu(boolean contextmenu) {
        this.contextmenu = contextmenu;
    }

    public Map<String, Object> getExtra() {
        return extra;
    }

    public void setExtra(Map<String, Object> extra) {
        this.extra = extra;
    }

    public Integer getParentId() {
        return parentId;
    }

    public void setParentId(Integer parentId) {
        this.parentId = parentId;
    }
}
